<?php

namespace Teknisa\FIS\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;


class Fis22000PreValidacaoDeclaracoesAcessoriasFactory {

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }


    public  function validarSped($cdimposto, $cdempresa, $nrsequmapapu, $inscricao, $idorgarrecad, $cdlivromag, $idconsprodest){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'PRE_VALIDACAO_SPED');
        $procedure->addParam(new Param(
                "P_CDIMPOSTO",
                Param::PARAM_INPUT,
                $cdimposto,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDEMPRESA",
                Param::PARAM_INPUT,
                $cdempresa,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_NRSEQUMAPAPU",
                Param::PARAM_INPUT,
                $nrsequmapapu,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_INSCRICAO",
                Param::PARAM_INPUT,
                $inscricao,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_IDORGARRECAD",
                Param::PARAM_INPUT,
                $idorgarrecad,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDLIVROMAG",
                Param::PARAM_INPUT,
                $cdlivromag,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_PRODESTOQUE",
                Param::PARAM_INPUT,
                $idconsprodest,
                Param::PARAM_TYPE_STR
        ));
        
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }

    public  function validarDapiMG($cdimposto, $cdempresa, $nrsequmapapu, $inscricao, $idorgarrecad, $cdlivromag){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'PRE_VALIDACAO_DAPI_MG');
        $procedure->addParam(new Param(
                "P_CDIMPOSTO",
                Param::PARAM_INPUT,
                $cdimposto,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDEMPRESA",
                Param::PARAM_INPUT,
                $cdempresa,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_NRSEQUMAPAPU",
                Param::PARAM_INPUT,
                $nrsequmapapu,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_INSCRICAO",
                Param::PARAM_INPUT,
                $inscricao,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_IDORGARRECAD",
                Param::PARAM_INPUT,
                $idorgarrecad,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDLIVROMAG",
                Param::PARAM_INPUT,
                $cdlivromag,
                Param::PARAM_TYPE_STR
        ));
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }
    public  function validarEfdReinf($cdimposto, $cdempresa, $nrsequmapapu, $inscricao, $idorgarrecad, $cdlivromag){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'PRE_VALIDACAO_REINF');
        $procedure->addParam(new Param(
                "P_CDIMPOSTO",
                Param::PARAM_INPUT,
                $cdimposto,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDEMPRESA",
                Param::PARAM_INPUT,
                $cdempresa,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_NRSEQUMAPAPU",
                Param::PARAM_INPUT,
                $nrsequmapapu,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_INSCRICAO",
                Param::PARAM_INPUT,
                $inscricao,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_IDORGARRECAD",
                Param::PARAM_INPUT,
                $idorgarrecad,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDLIVROMAG",
                Param::PARAM_INPUT,
                $cdlivromag,
                Param::PARAM_TYPE_STR
        ));
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }

    public  function validarSpedContribuicoes($cdempresa, $dtinicial, $dtfinal, $cdlivromag){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'PRE_VALIDACAO_SPED_CONTRIB');
        $procedure->addParam(new Param(
                "P_CDEMPRESA",
                Param::PARAM_INPUT,
                $cdempresa,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_DTINICIAL",
                Param::PARAM_INPUT,
                $dtinicial,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_DTFINAL",
                Param::PARAM_INPUT,
                $dtfinal,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_CDLIVROMAG",
                Param::PARAM_INPUT,
                $cdlivromag,
                Param::PARAM_TYPE_STR
        ));
        
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }

    public function buscaNotasContribuicoes($dtinicial, $dtfinal, $cdempresa, $erros, $dstipovalida) {
        #sets parameter data
        $params                   = array();
        $params['P_DTFINAL']      = $dtfinal;
        $params['P_DTINICIAL']    = $dtinicial;
        $params['P_CDEMPRESA']    = $cdempresa;
        $params['P_ERROS']        = $erros;
        $params['P_DSTIPOVALIDA'] = $dstipovalida;
        #defines function query
        $sql = "SELECT DISTINCT N.CDFILIAL,
                    N.NRLANCTONF,
                    N.NRNOTAFISC AS NRNOTAFISCNFPRE,
                    DECODE(N.IDORIGDESTNF,
                    'C', C.NRINSJURCLIE, DECODE(N.IDORIGDESTNF, 'F', F.NRINSJURFORN, DECODE(N.IDORIGDESTNF, 'L', L.CDFILIAL, DECODE(N.CDFORNECED, NULL, DECODE(N.CDCLIENTE, NULL,L.CDFILIAL,C.NRINSJURCLIE),F.NRINSJURFORN)))) AS CDDESTEMIT, DECODE(N.IDORIGDESTNF, 'C', C.NMRAZSOCCLIE, DECODE(N.IDORIGDESTNF, 'F', F.NMRAZSOCFORN, DECODE(N.IDORIGDESTNF, 'L',L.NMRAZSOCFILI, DECODE(N.CDFORNECED, NULL, DECODE(N.CDCLIENTE, NULL,L.NMRAZSOCFILI,C.NMRAZSOCCLIE),F.NMRAZSOCFORN)))) AS NMDESTEMIT, N.DTEMISSAO AS DTEMISSAONFPRE, N.DTENTRSAID AS DTENTRSAIDNFPRE, N.IDENTRSAIDOP AS IDENTRSAIDNFPRE, N.VRNOTAFISC AS VRNOTAFISCNFPRE, N.CDINSCESTA, N.CDINSCMUNI, N.IDTPIJURNF, N.CDTIPOOPER, DECODE(N.IDENTRSAIDOP, 'E', N.CDSERIEENTR, N.CDSERIE) AS CDSERIE, N.IDVERIFNFCONF AS IDSTATUS, N.IDORIGDESTNF
                FROM NOTAFISCAL N, FORNECEDOR F, CLIENTE C, FILIAL L, ERRONFPREVALID E
                WHERE N.CDFILIAL = E.CDFILIAL
                        AND N.NRLANCTONF = E.NRLANCTONF
                        AND (E.IDERROAVISNFPRE = :P_ERROS
                        OR :P_ERROS = 'T')
                        AND E.DSTIPOVALIDA = :P_DSTIPOVALIDA
                        AND N.CDFORNECED = F.CDFORNECED(+)
                        AND N.CDCLIENTE = C.CDCLIENTE(+)
                        AND N.CDEMPRESA = :P_CDEMPRESA
                        AND DECODE(N.IDENTRSAIDOP,'E',N.DTENTRSAID,N.DTEMISSAO)
                    BETWEEN :P_DTINICIAL
                        AND :P_DTFINAL
                        AND DECODE(N.IDENTRSAIDOP,'E',N.CDFILIEMIT,N.CDFILIRECE) = L.CDFILIAL(+)
                ORDER BY  N.NRNOTAFISC";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function buscaNotas($cdimposto, $nrsequmapapu, $erros, $dstipovalida) {
        #sets parameter data
        $params                   = array();
        $params['P_CDIMPOSTO']    = $cdimposto;
        $params['P_NRSEQUMAPAPU'] = $nrsequmapapu;
        $params['P_ERROS']        = $erros;
        $params['P_DSTIPOVALIDA'] = $dstipovalida;
        #defines function query
        $sql = "SELECT DISTINCT N.CDFILIAL,
                    N.NRLANCTONF,
                    N.NRNOTAFISC AS NRNOTAFISCNFPRE,
                    DECODE(N.IDORIGDESTNF,
                    'C', C.NRINSJURCLIE, DECODE(N.IDORIGDESTNF, 'F', F.NRINSJURFORN, DECODE(N.IDORIGDESTNF, 'L', L.CDFILIAL, DECODE(N.CDFORNECED, NULL, DECODE(N.CDCLIENTE, NULL,L.CDFILIAL,C.NRINSJURCLIE),F.NRINSJURFORN)))) AS CDDESTEMIT, DECODE(N.IDORIGDESTNF, 'C', C.NMRAZSOCCLIE, DECODE(N.IDORIGDESTNF, 'F', F.NMRAZSOCFORN, DECODE(N.IDORIGDESTNF, 'L',L.NMRAZSOCFILI, DECODE(N.CDFORNECED, NULL, DECODE(N.CDCLIENTE, NULL,L.NMRAZSOCFILI,C.NMRAZSOCCLIE),F.NMRAZSOCFORN)))) AS NMDESTEMIT, N.DTEMISSAO AS DTEMISSAONFPRE, N.DTENTRSAID AS DTENTRSAIDNFPRE, N.IDENTRSAIDOP AS IDENTRSAIDNFPRE, N.VRNOTAFISC AS VRNOTAFISCNFPRE, N.CDINSCESTA, N.CDINSCMUNI, N.IDTPIJURNF, N.CDTIPOOPER, DECODE(N.IDENTRSAIDOP, 'E', N.CDSERIEENTR, N.CDSERIE) AS CDSERIE, N.IDVERIFNFCONF AS IDSTATUS, N.IDORIGDESTNF
                FROM IMPOSTONF I, IMPOSTO IM, NOTAFISCAL N, FORNECEDOR F, CLIENTE C, FILIAL L, ERRONFPREVALID G
                WHERE I.CDIMPOSTO = IM.CDIMPOSTO
                        AND IM.CDIMPOSTONFE = :P_CDIMPOSTO
                        AND I.NRSEQUMAPAPU = :P_NRSEQUMAPAPU
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND N.CDFILIAL = G.CDFILIAL
                        AND N.NRLANCTONF = G.NRLANCTONF
                        AND G.IDERROAVISNFPRE = NVL(:P_ERROS,G.IDERROAVISNFPRE)
                        AND G.DSTIPOVALIDA = :P_DSTIPOVALIDA
                        AND N.CDFORNECED = F.CDFORNECED(+)
                        AND N.CDCLIENTE = C.CDCLIENTE(+)
                        AND DECODE(N.IDENTRSAIDOP, 'E', N.CDFILIEMIT, N.CDFILIRECE) = L.CDFILIAL(+)
                ORDER BY  N.NRNOTAFISC";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function buscaErros($nrsequmapapu, $erros, $dstipovalida) {
        #sets parameter data
        $params                   = array();
        $params['P_NRSEQUMAPAPU'] = $nrsequmapapu;
        $params['P_ERROS']        = $erros;
        $params['P_DSTIPOVALIDA'] = $dstipovalida;
        #defines function query
        $sql = "SELECT DISTINCT N.DTEMISSAO,
                    'Número NF: ' ||N.NRNOTAFISC || ' - Identificador: ' || decode(N.IDENTRSAIDOP,'E','Entrada','Saída') || ' - Data de Emissão: '|| to_char(N.DTEMISSAO,'DD/MM/YYYY') || ' - Data de Entrada/Saída: ' || to_char(N.DTENTRSAID,'DD/MM/YYYY') AS DESCRICAO, E.NRSEQERRONFPRE, E.DSERRONFPRE, E.IDERRONFPRE, E.IDERROAVISNFPRE
                FROM ERRONFPREVALID E, NOTAFISCAL N, IMPOSTONF I, FORNECEDOR F, CLIENTE C, FILIAL L
                WHERE E.IDERROAVISNFPRE = NVL(:P_ERROS,E.IDERROAVISNFPRE)
                        AND E.DSTIPOVALIDA = :P_DSTIPOVALIDA
                        AND E.CDFILIAL = N.CDFILIAL
                        AND E.NRLANCTONF = N.NRLANCTONF
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND I.NRSEQUMAPAPU = :P_NRSEQUMAPAPU
                        AND N.CDFORNECED = F.CDFORNECED(+)
                        AND N.CDCLIENTE = C.CDCLIENTE(+)
                        AND DECODE(N.IDENTRSAIDOP, 'E', N.CDFILIEMIT, N.CDFILIRECE) = L.CDFILIAL(+)
                        AND E.NRORG = N.NRORG
                        AND I.NRORG = N.NRORG
                ORDER BY  NRSEQERRONFPRE";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function buscaErrosContribuicoes($cdempresa, $dtinicial, $dtfinal, $erros, $dstipovalida) {
        #sets parameter data
        $params                   = array();
        $params['P_CDEMPRESA']    = $cdempresa;
        $params['P_DTINICIAL']    = $dtinicial;
        $params['P_DTFINAL']      = $dtfinal;
        $params['P_ERROS']        = $erros;
        $params['P_DSTIPOVALIDA'] = $dstipovalida;
        #defines function query
        $sql = "SELECT DISTINCT N.DTEMISSAO,
                    'Número NF: ' ||N.NRNOTAFISC || ' - Identificador: ' || decode(N.IDENTRSAIDOP,'E','Entrada','Saída') || ' - Data de Emissão: '|| to_char(N.DTEMISSAO,'DD/MM/YYYY') || ' - Data de Entrada/Saída: ' || to_char(N.DTENTRSAID,'DD/MM/YYYY') AS DESCRICAO, E.NRSEQERRONFPRE, E.DSERRONFPRE, E.IDERRONFPRE, E.IDERROAVISNFPRE
                FROM ERRONFPREVALID E, NOTAFISCAL N, FORNECEDOR F, CLIENTE C, FILIAL L
                WHERE E.IDERROAVISNFPRE = NVL(:P_ERROS,E.IDERROAVISNFPRE)
                        AND E.DSTIPOVALIDA = :P_DSTIPOVALIDA
                        AND E.CDFILIAL = N.CDFILIAL
                        AND E.NRLANCTONF = N.NRLANCTONF
                        AND N.CDFORNECED = F.CDFORNECED(+)
                        AND N.CDCLIENTE = C.CDCLIENTE(+)
                        AND DECODE(N.IDENTRSAIDOP,'E',N.DTENTRSAID,N.DTEMISSAO)
                    BETWEEN :P_DTINICIAL
                        AND :P_DTFINAL
                        AND N.CDEMPRESA = :P_CDEMPRESA
                        AND DECODE(N.IDENTRSAIDOP,'E',N.CDFILIEMIT,N.CDFILIRECE) = L.CDFILIAL(+)
                        AND E.NRORG = N.NRORG
                ORDER BY  NRSEQERRONFPRE";


        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

}