<?php

namespace Teknisa\FIS\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;


class Fis57000FichaConteudoImportacaoFactory {

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function updateValores($vrparcimpo, $vrvendaie, $cdfilial, $nrmapafci, $cdprodutoop){
      	$sql = "UPDATE ITMAPAFCI
                   SET VRPARCIMPO = ROUND(:P_VRPARCIMPO, 2),
                       VRVENDAIE = ROUND(:P_VRVENDAIE, 2),
                       VRFCI = ROUND((ROUND(:P_VRPARCIMPO, 2)/ROUND(:P_VRVENDAIE, 2))*100, 2)
                 WHERE CDFILIALFCI = :P_CDFILIAL
                   AND NRMAPAFCI = :P_NRMAPAFCI
                   AND CDPRODUTOOP = :P_CDPRODUTOOP";
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        
        $params = array('P_VRPARCIMPO' => $vrparcimpo,
                        'P_VRVENDAIE' => $vrvendaie,
                        'P_CDFILIAL' => $cdfilial,
                        'P_NRMAPAFCI' => $nrmapafci,
                        'P_CDPRODUTOOP' => $cdprodutoop);
                        
        $std->execute($params);
    }
    
    public function updateIdsitmapafci($idsitmapafci, $cdfilial, $nrmapafci){
      	$sql = "UPDATE MAPAFCI
                   SET IDSITMAPAFCI = :P_IDSITMAPAFCI
                 WHERE CDFILIALFCI = :P_CDFILIAL
                   AND NRMAPAFCI = :P_NRMAPAFCI";
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        
        $params = array('P_IDSITMAPAFCI' => $idsitmapafci,
                        'P_CDFILIAL' => $cdfilial,
                        'P_NRMAPAFCI' => $nrmapafci);
                        
        $std->execute($params);
    }
    
    public function salvarRecibo($nrctrlfci, $cdfilialfci, $cdprodutoop, $nrloteestq, $nrsublote){
      	$sql = "UPDATE LOTE
                   SET NRCTRLFCI = :P_NRCTRLFCI
                 WHERE CDFILIAL = :P_CDFILIALFCI
                   AND CDPRODUTO = :P_CDPRODUTOOP
                   AND NRLOTEESTQ = :P_NRLOTEESTQ
                   AND NRSUBLOTE = :P_NRSUBLOTE";
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        
        $params = array('P_NRCTRLFCI' => $nrctrlfci,
                        'P_CDFILIALFCI' => $cdfilialfci,
                        'P_CDPRODUTOOP' => $cdprodutoop,
                        'P_NRLOTEESTQ' => $nrloteestq,
                        'P_NRSUBLOTE' => $nrsublote);
                        
        $std->execute($params);
    }
    
    public  function apurarPeriodoFci($cdfilialfci, $nrmapafci){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'APURA_FCI');
        $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $cdfilialfci,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_NRMAPAFCI",
                Param::PARAM_INPUT,
                $nrmapafci,
                Param::PARAM_TYPE_STR
        ));
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }
    
    public  function geraDadosFci($cdfilialfci, $nrmapafci){
        $procedure = new StoredProcedure($this->entityManager->getConnection(),'GERA_FCI');
        $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $cdfilialfci,
                Param::PARAM_TYPE_STR
        ));
        $procedure->addParam(new Param(
                "P_NRMAPAFCI",
                Param::PARAM_INPUT,
                $nrmapafci,
                Param::PARAM_TYPE_STR
        ));
        $retornoProcedure = $procedure->execute();
        return $retornoProcedure;
    }
    
    public function buscaLinhasArquivo($cdfilialfci, $nrmapafci) {
        $sql = "SELECT  DSLINHA
                  FROM ARQUIVOTXT
                 WHERE CDFILIALFCI = :P_CDFILIAL
                   AND NRMAPAFCI = :P_NRMAPAFCI
              ORDER BY NRBLOCO, NRLINHA";
                
        $std = $this->connection->prepare($sql);
        
        $params = array('P_CDFILIAL' => $cdfilialfci,
                        'P_NRMAPAFCI' => $nrmapafci);
                        
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function validaPeriodoMapa($dtiniperiodo, $dtfimperiodo, $cdfilialfci) {
        $sql = "SELECT M.CDFILIALFCI, M.NRMAPAFCI, M.DTINIPERIODO, M.DTFIMPERIODO
                  FROM MAPAFCI M, (SELECT DTDATA
                                     FROM DATA
                                    WHERE DTDATA BETWEEN :P_DTINIPERIODO AND :P_DTFIMPERIODO) D
                 WHERE M.CDFILIALFCI = :P_CDFILIAL
                   AND D.DTDATA BETWEEN M.DTINIPERIODO AND M.DTFIMPERIODO";
                
        $std = $this->connection->prepare($sql);
        
        $params = array('P_DTINIPERIODO' => $dtiniperiodo,
                        'P_DTFIMPERIODO' => $dtfimperiodo,
                        'P_CDFILIAL' => $cdfilialfci);
                        
        $std->execute($params);
        
        return $std->fetchAll();
    }
}