<?php

namespace Teknisa\FIS\Modules\Guias\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Modules\Guias\Factories\Fis00200DocumentoDeArrecadacaoFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\Address;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIS\Utilities\MessagesFiscal;
use Zeedhi\Framework\DTO\Row;

class Fis00200DocumentoDeArrecadacaoService {

    const ENVIAR_LOTE_PROD = 'https://www.gnre.pe.gov.br/gnreWS/services/GnreLoteRecepcao';
    const ENVIAR_LOTE_TEST = 'https://www.testegnre.pe.gov.br/gnreWS/services/GnreLoteRecepcao';
    const CONSULTA_LOTE_PROD = 'https://www.gnre.pe.gov.br/gnreWS/services/GnreResultadoLote';
    const CONSULTA_LOTE_TEST = 'https://www.testegnre.pe.gov.br/gnreWS/services/GnreResultadoLote';
    
    const ENVIAR_ES_LOTE_TEST = 'https://homologacao.sefaz.es.gov.br/WsDua';
    const ENVIAR_ES_LOTE_PROD = 'https://app.sefaz.es.gov.br/WsDua';
    
    protected $dataSourceManager;
    protected $entityManager;
    protected $environment;
    protected $factory;
    protected $newCode;
    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $crudService;
    
    private $_certFile;
    private $_certPass;
    private $_certKeyFile;
    
    //
        
    
    public function __construct($crudService, $dataSourceManager, \Doctrine\ORM\EntityManager $entityManager, Environment $environment, NewCode $newCode ,Fis00200DocumentoDeArrecadacaoFactory $factory, $fileServerUrl, $fileUploader) {
        $this->crudService              = $crudService;
        $this->dataSourceManager        = $dataSourceManager;
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->factory                  = $factory;
        $this->newCode                  = $newCode;
        $this->nrorg                    = $environment->getNrorgTrab();
        $this->operador                 = $this->environment->getCdOperador();
        $this->currentTime              = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->fileUploader             = $fileUploader;
        $this->_fileServerUrl           = $fileServerUrl;
        
        $this->_tempCertificatesPath    = DIRECTORY_SEPARATOR.'tmp'. DIRECTORY_SEPARATOR . "certificates" . DIRECTORY_SEPARATOR . $this->nrorg;
    
    }
   
   
   
    #############################################################
    #                                                           #
    #              Funções  de Processo                         #
    #                                                           #
    #############################################################
    
    public function salvarEdicaoGuia($row) {
        try{
            $this->validaMapaBloqueado($row);  
            
            // Verifica se ja existe o registro na tabela de Pagamento da Guia para ver se vai inserir ou atualizar
            $documento = $this->factory->getDocarrecadpag($row);
            if($documento) {
                $this->factory->updateDocarrecadpag(
                    isset($row['CDNATUREZA']) ? $row['CDNATUREZA'] : null,
                    isset($row['CDRECEITA']) ? $row['CDRECEITA'] : null,
                    isset($row['CDDETRECEITA']) ? $row['CDDETRECEITA'] : null,
                    $row['DTVENCIMENTO'], 
                    isset($row['CDFILIALPAG'])   ? $row['CDFILIALPAG']   : null, 
                    isset($row['NRLANCTONFPAG']) ? $row['NRLANCTONFPAG'] : null,
                    $documento['NRSEQDOCARRECAD'],
                    $row['VRMULTA'],
                    $row['VRJUROS'],
                    $row['VRGUIA'],
                    $row['DTLIMITEPAG'],
                    $row['NRSEQGUIACOMPL'],
                    isset($row['NMMAPAAPUR'])    ? $row['NMMAPAAPUR']    : null,
                    $row['IDTIPOGUIA'],
                    $row['NRSEQUMAPAPU'],
                    isset($row['NRSEQGUIAAPURNC']) ? $row['NRSEQGUIAAPURNC'] : null, //GUIAAPURMAPANC
                    isset($row['NRSEQMAPAPURESTNC']) ? $row['NRSEQMAPAPURESTNC'] : null
                );
                return $documento['NRSEQDOCARRECAD'];
            } else {
                $params = array(
                    "NRSEQDOCARRECAD" => $this->newCode->getCodeByTableName('DOCARRECADPAG', $this->nrorg,1,20),
                    "NRSEQUMAPAPUSN"  => $row['IDTIPOGUIA'] == 'S' ? $row['NRSEQUMAPAPU'] : null,
                    "NRSEQUMAPAPU"    => $row['IDTIPOGUIA'] != 'S' ? $row['NRSEQUMAPAPU'] : null,
                    
                    "NRSEQGUIAAPURNC" => isset($row['NRSEQGUIAAPURNC']) ? $row['NRSEQGUIAAPURNC'] : null , //GUIAAPURMAPANC
                    "NRSEQMAPAPURESTNC" => isset($row['NRSEQMAPAPURESTNC']) ? $row['NRSEQMAPAPURESTNC'] : null,
                    
                    "NRSEQGUIACOMPL"  => isset($row['NRSEQGUIACOMPL']) ? $row['NRSEQGUIACOMPL'] : null,
                    "CDFILIAL"        => isset($row['CDFILIAL']     ) ? $row['CDFILIAL']      : null,
                    "CDFILIALPAG"     => isset($row['CDFILIALPAG']  ) ? $row['CDFILIALPAG']   : null,
                    "NRLANCTONF"      => isset($row['NRLANCTONF']   ) ? $row['NRLANCTONF']    : null,
                    "NRLANCTONFPAG"   => isset($row['NRLANCTONFPAG']) ? $row['NRLANCTONFPAG'] : null,
                    "CDIMPOSTO"       => $row['CDIMPOSTO'], 
                    "DTVENCIMENTO"    => $row['DTVENCIMENTO'],
                    "CDNATUREZA"      => isset($row['CDNATUREZA']) ? $row['CDNATUREZA'] : null,
                    "CDRECEITA"       => isset($row['CDRECEITA']) ? $row['CDRECEITA'] : null,
                    "CDDETRECEITA"    => isset($row['CDDETRECEITA']) ? $row['CDDETRECEITA'] : null,
                    "IDTIPOGUIA"      => $row['IDTIPOGUIA'],
                    "NRORG"           => $this->nrorg,
                    "VRMULTA"         => $row['VRMULTA'],
                    "VRJUROS"         => $row['VRJUROS'],
                    "VRPRINCIPAL"     => $row['VRGUIA'],
                    "DTLIMITEPAG"     => $row['DTLIMITEPAG'],
                    "NMMAPAAPUR"      => isset($row['NMMAPAAPUR']   ) ? $row['NMMAPAAPUR']    : null
                );
                $this->factory->insertDocarrecadpag($params);
                return $params['NRSEQDOCARRECAD'];
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function preparaPagamento ($row){
        try {
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            //Row referente a current row da guia que se deseja emitir.
            $guias = $row['row'];
            
            $response = [];
            
            // (widget.currentRow.IDBLOQMAPAPU === 'S' && widget.currentRow.IDSTATBPAG === 'P')
            
            // Row referente aos dados de pagamento(CDFILIAL e CDRATEIO)
            $dadosPagameto = $row['dadosPagamento'];
            
            $info = $row['info'];
            
            foreach ($guias as $guia) {
                
                if (($guia['IDSTATBPAG'] === 'P' && empty($guia['IDBLOQMAPAPU'])) || ($guia['IDBLOQMAPAPU'] === 'S' && $guia['IDSTATBPAG'] === 'P')) {
                    $status = $this->factory->getBoleto($guia['CDFILIALPAG'], $guia['NRLANCTONFPAG']);
                    if($status) {
                        // throw new \Exception(MessagesFiscal::PREPARACAO_REALIZADA,1);
                        array_push($response, $guia['NRSEQLOTE'].' - Preparação de Pagamento já foi realizada.');
                    } else {
                        if($info['IDTIPOEMISSAO'] === 'A') {
                            $this->validaMapaBloqueado($guia);  
                        }
                    }
                } 
                else {
                    // Implementar algo para qual guia nao houve a preparação de pagamento    
                    array_push($response, $guia['NRSEQLOTE'].' - Não foi possivel realizar a preparação de pagamento');
                }
            }
            
            $parametrosFiscal = $this->factory->getParametrosFiscal();
            // VErificar futuramente a parametrização do simples nacional;
            $parametrosGuia   = $this->factory->getParametrosEmissaoGuia($info['CDIMPOSTO']);

            if(!$parametrosFiscal) 
                throw new \Exception(MessagesFiscal::PARMAFISCAL_VAZIO,1);
            if($parametrosFiscal['IDGERCONPAGGUIA'] ==='N')
                throw new \Exception(MessagesFiscal::NAO_GERACONTA,1);
            if(!$parametrosFiscal['CDTIPOOPERPREPAG'])
                throw new \Exception(MessagesFiscal::TIPOOPERCAO_VAZIO,1);    
            if(!$parametrosGuia)
                throw new \Exception(MessagesFiscal::PARAMETROSIMPOSTO_VAZIO,1);
            
            $dtFimPeriodo;
            
            // Parametrização de Pagamento(CDFILIAL e CDRATEIO)
            foreach ($guias as $guia) {
                
                if (($guia['IDSTATBPAG'] === 'P' && empty($guia['IDBLOQMAPAPU'])) || ($guia['IDBLOQMAPAPU'] === 'S' && $guia['IDSTATBPAG'] === 'P')) {
                    if($info['IDTIPOEMISSAO'] === 'D') {
                        if($guia['OPERACAO_DOC'] === 'E') {
                           $dtFimPeriodo = $guia['DTENTRSAID_DOC'];
                        } else if($guia['OPERACAO_DOC'] === 'S') {
                           $dtFimPeriodo = $guia['DTEMISSAO_DOC'];
                        }
                    } else if($info['IDTIPOEMISSAO'] === 'A') {
                        $dtFimPeriodo = $guia['DTFIMPERIODO'];
                    }
                    
                    if(($guia['IDTRANSMISSAO'] === 'S') && (count($guias) > 1)) {
                        $valorTotal = 0;
                        $arrayDtPagamento = [];
                        foreach ($guias as $sumGuia) {
                            $valorTotal = $valorTotal + round($sumGuia['VRTOTAL'],2);
                            array_push ($arrayDtPagamento, $sumGuia ['DTLIMITEPAG']);
                        }
                        
                        $arrayDtPagamento = array_unique($arrayDtPagamento);
                        $info['VRTOTAL'] = $valorTotal;
                        $info['DTFIMPERIODO'] = $dtFimPeriodo;
                        $this->verificaDadosPagamento($info, $dadosPagameto); 
                        foreach($arrayDtPagamento as $dtpagamento) {
                            $info['DTLIMITEPAG'] = $dtpagamento;
                            $this->geraFinanceiroPorLote($guias, $dadosPagameto, $parametrosGuia, $parametrosFiscal, $info);
                        }
                        
                    } else {
                        if (count($guias) > 1) {
                            foreach ($guias as $guia) {
                                $guia['CDEMPRESA'] = $info['CDEMPRESA'];
                                $guia['DTFIMPERIODO'] = $dtFimPeriodo;
                                $this->verificaDadosPagamento($info, $dadosPagameto); 
                                $this->geraFinanceiro($guia, $dadosPagameto, $parametrosGuia, $parametrosFiscal, $info);
                            }
                        } else {
                            $guia['CDEMPRESA'] = $info['CDEMPRESA'];
                            $guia['DTFIMPERIODO'] = $dtFimPeriodo;
                            $this->verificaDadosPagamento($info, $dadosPagameto); 
                            $this->geraFinanceiro($guia, $dadosPagameto, $parametrosGuia, $parametrosFiscal, $info);        
                        }
                    }
                }
            }
            
            $connection->commit();
            
            return array_unique($response);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function cancelaDadosFinanceiro ($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $guias = $row['row'];
            
            $response = [];
            
            $info = $row['info']; 
            foreach($guias as $guia) {
                if ((($guia['IDSTATBPAG'] === 'B' || $guia['IDSTATBPAG'] === 'A') && empty($guia['IDBLOQMAPAPU'])) || 
                    ((isset($guia['IDBLOQMAPAPU']) && $guia['IDBLOQMAPAPU'] === 'S') && ($guia['IDSTATBPAG'] === 'B' || $guia['IDSTATBPAG'] === 'A'))) {
                    
                    $status = $this->factory->getBoleto($guia['CDFILIALPAG'], $guia['NRLANCTONFPAG']);
        
                    if(!$status) {
                        // throw new \Exception(MessagesFiscal::PREPARACAO_NAO_REALIZADA,1);
                        array_push($response, $guia['NRSEQLOTE'].' - Preparação de Pagamento não foi já foi realizada.');
                    } else {
                        $this->validaMapaBloqueado($guia);  
                   
                        // VErificar futuramente a parametrização do simples nacional;
                        $parametrosGuia   = $this->factory->getParametrosEmissaoGuia($info['CDIMPOSTO']);
                        if(!$parametrosGuia)
                            throw new \Exception(MessagesFiscal::PARAMETROSIMPOSTO_VAZIO,1);
                            
                        $params = array(
                            "P_CDFILIAL"   => $guia["CDFILIALPAG"],
                            "P_NRLANCTONF" => $guia["NRLANCTONFPAG"],
                            "P_IDCANFIN"   => 'S'
                            
                        );
                        $retorno = $this->factory->cancelaDadosFinanceiro($params);
        
                        if($retorno ==='0'){
                            $notaFiscal    =  $this->factory->getNotaFiscal($guia['CDFILIALPAG'], $guia['NRLANCTONFPAG']);
                            
                            $params = array(
                                "CDFILIAL"   => $guia["CDFILIALPAG"],
                                "NRLANCTONF" => $guia["NRLANCTONFPAG"]
                            );
            
                            $this->salvarEdicaoGuia(array_merge($guia, array('NRLANCTONFPAG' => null, 'CDFILIALPAG' => null  )));
                            $this->entityManager->flush();
                            
                            $this->factory->deleteContaPagar($params);
                            $this->entityManager->flush();
                            
                            $this->factory->deleteNotaFiscal($params);
                            $this->entityManager->flush();
                            
                            $movfluxo = $this->atualizaFluxoCaixa($this->nrorg, $info['CDEMPRESA'], $notaFiscal['CDFILIRECE'], $parametrosGuia['CDCLASSFINA'], $guia['DTVENCIMENTO'], round($guia['VRTOTAL'],2), 'E', 'S' );
                            $this->entityManager->persist($movfluxo);
                            $this->entityManager->flush();
                        
                        }
                        
                        else{
                            switch ($retorno) {
                                case '1':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO1,1);
                                    break;
                                case '2':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO2,1);
                                    break;
                                case '4':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO4,1);
                                break;
                                case '5':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO5,1);
                                    break;
                                case '6':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO6,1);
                                    break;
                                case '7':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO7,1);
                                    break;
                                case '8':
                                    throw new \Exception(MessagesFiscal:: ERROCANCELAMENTO8,1);
                                    break;
                            }
                        }
                    }
                } else {
                    array_push($response, $guia['NRSEQLOTE']);
                }
            }
            
            $connection->commit();
            
            return array_unique($response);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function transmitirLoteRj ($dataset){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $rows = $dataset[0];
            $info = $dataset[1];
            $datasPagamento = $dataset[2];
            
            foreach ($datasPagamento as $data){
                $documentsRows[$data] = [];
                foreach($rows as $row) {
                    if($row['DTLIMITEPAG'] === $data) {
                        array_push($documentsRows[$data], $row);
                    }
                    if(!empty($row['NRSEQLOTE'])) {
                        $this->factory->updateDocarrecadpagNewLote($row['NRSEQLOTE']);
                        $this->entityManager->flush();
                    }
                }
            }
            
            $response = [];
            
            foreach ($datasPagamento as $data) {
             // var_dump($rows, $info, $datasPagamento);
                $xml       = $this->getXmlTransmissaoLoteRj($documentsRows[$data], $info);
                // var_dump($xml);die;
                $resultado = $this->conectaWebService($xml,"http://www1.fazenda.rj.gov.br/projetoGCTBradesco/br/gov/rj/sef/gct/webservice/GerarDocumentoArrecadacaoWS.jws","http://www7.fazenda.rj.gov.br" );
                $retorno   = $this->trataRetornoEnvioLoteRj($resultado, $documentsRows[$data], $info);
                
                array_push($response, $retorno);
            }  
            $connection->commit();
            return $response;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function consultarLoteRj ($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $xml       = $this->getXmlConsultaLoteRj($row);  
            $resultado = $this->conectaWebService($xml,"http://www1.fazenda.rj.gov.br/projetoGCTBradesco/br/gov/rj/sef/gct/webservice/GerarDocumentoArrecadacaoWS.jws","http://www7.fazenda.rj.gov.br" );
            $retorno   = $this->trataRetornoConsultaLoteRj($resultado, $row);
           
            $connection->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function consultarLoteGnreOnline ($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $params = array(
                "CDEMPRESA"     => $row["CDEMPRESA"],
                "CDINSCESTA"     => $row["CDINSCESTA"]
            );
            
            $parametro = $this->factory->getParamtrans($params);
            
            $xml       = $this->getXmlConsultaLoteGnreOnline($row, $parametro);  
            
            $this->setCertFileKey($row, $parametro);
            
            $resultado = $this->conectaWebServiceGnreOnline($xml, ($parametro['IDAMBIENTE'] === '1' ? self::CONSULTA_LOTE_PROD : self::CONSULTA_LOTE_TEST), "http://www.gnre.pe.gov.br" );
            
            $retorno   = $this->trataRetornoConsultaLoteGnreOnline($resultado, $row);
           
            $connection->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
   
    public function transmitirLoteEs ($dataset){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $rows = $dataset[0];
            $info = $dataset[1];

            $params = array(
                "CDEMPRESA"     => $info["CDEMPRESA"],
                "CDINSCESTA"     => $info["CDINSCESTA"]
            );
            
            $parametro = $this->factory->getParamtrans($params);
            
            $response = [];
            
            $result = $this->getXmlEmissaoGuiaEspiritoSanto($rows, $info, $parametro);
            
            var_dump($result);die;
            // $xml       = $this->getXmlTransmissaoLoteEs($rows, $info, $parametro);
            // var_dump($xml);die;
            // $this->setCertFileKey($info, $parametro);
            
            // https://homologacao.sefaz.es.gov.br/WsDua
            // https://app.sefaz.es.gov.br/WsDua
            // $resultado = $this->conectaWebServiceGnreOnline($xml, $parametro['IDAMBIENTE'] === '1' ? self::ENVIAR_ES_LOTE_PROD : self::ENVIAR_ES_LOTE_TEST, "http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao" );
            // $resultado = $this->conectaWebServiceGnreOnline($xml, "https://app.sefaz.es.gov.br/WsDua?wsdl", "http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao" );
            
            array_push($response, $retorno);
            
            $connection->commit();
            return $response;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    #############################################################
    #                                                           #
    #              Funções  de Validação                        #
    #                                                           #
    #############################################################



    /*
     * Valida impostos relacionados com impostos NF-e, verificando se seus
     * atributos são iguais.
     */
    public function validaImpostosRelacionados($cdimpostoNfe) {
        // Busca os dados do Imposto NF-e.
        $params = array(
            "NRORG" => $this->nrorg,
            "CDIMPOSTONFE" => $cdimpostoNfe
        );
        $impostoNfe = $this->factory->getImpostoNfe($params);
        // Busca os impostos relacionados ao código de imposto NF-e.
        $impostosRelac = $this->factory->getImpostoRelac($params);

        // Se as duas tabelas possuem dados, em um primeiro momento a validação
        // ocorreu com sucesso.
        if (($impostosRelac) && ($impostoNfe)) {
            $paramRelacionamento = true;
        }
        else {
            $paramRelacionamento = false;
        }

        // Caso encontre algum imposto onde haja divergência na parametrização,
        // a validação deve retornar falso, para permitir o tratamento no
        // frontend.
        if ($paramRelacionamento) {
            foreach ($impostosRelac as $imposto) {
                if (($imposto['IDORGARRECAD'] != $impostoNfe['IDORGARRECADNFE']) ||
                    ($imposto['IDFORMAPUR']   != $impostoNfe['IDFORMAPURNFE'])   ||
                    ($imposto['IDIMPORECUP']  != $impostoNfe['IDIMPORECUPNFE'])) {
                    $paramRelacionamento = false;
                }
            }
        }

        $validation = array(
            "VDPARAMIMPRELAC" => $paramRelacionamento
        );
        return $validation;
    }

    /*
     * Verifica se um mapa de apuração está bloqueado, lançando exceção em caso
     * positivo. Essa função deve ser chamada em todas as situações em que um
     * procedimento não pode ser executado com o mapa bloqueado. Guias do tipo
     * documento não são verificadas
     */
    public function validaMapaBloqueado($row) {
        switch($row['IDTIPOGUIA']) {
            //Não há mapa para emissão através de uma Nota Fiscal (Documento)
            case 'I': //Imposto Normal
            case 'O': //Difal Não Contribuinte Origem
            case 'D': //Difal Não Contribuinte Destino
            case 'F': //Fundo de Combate a Pobreza Não Contribuinte
            case 'P': //FECP
            case 'A': //Diferencial de Alíquota
                return;
                break;
        }
        $params = array(
            "NRORG" => $this->nrorg,
            "NRSEQUMAPAPU" => $row["NRSEQUMAPAPU"]
        );
        $idbloqmapapu = $this->factory->getIdbloqmapapu($params);
        if ($idbloqmapapu != 'S') {
            throw new \Exception(MessagesFiscal::MAPA_NAO_BLOQUEADO,1);
            
        }
    }    
    
    /*
        Verifica os dados padrões de pagamento relacioando ao filtro realizado pelo usuário.
        Controle realiadado na tabela PARAMGUIAPAG, caso não exista nenhuma parametrização
        é inserido um novo registro (CDFILIAL, E CDRATEIO) se não atualiza o registro atual.
    */    
    public function verificaDadosPagamento($info, $dadosPagameto){
        $params = array(
            "IDTIPOEMISSAO" => $info["IDTIPOEMISSAO"],
            "CDEMPRESA"     => $info["CDEMPRESA"],
            "CDIMPOSTO"     => $info["CDIMPOSTO"]
        );
        
        $dados = $this->factory->getParamguiapag($params);
        if($dados){
             $this->factory->updateParamguiapag($dadosPagameto['CDFILIAL'], $dadosPagameto['CDRATEIO'], $dados['NRSEQDADOS']);
        }else{
            $params = array(
                "NRSEQDADOS"    => (float) $this->newCode->getCodeByTableName('PARAMGUIAPAG', $this->nrorg,1,20),
                "CDFILIAL"      => $dadosPagameto['CDFILIAL'],
                "CDRATEIO"      => isset($dadosPagameto['CDRATEIO']) ? $dadosPagameto['CDRATEIO']   : null,
                "CDIMPOSTO"     => $info['CDIMPOSTO'] === 'SN'? null: $info['CDIMPOSTO'], 
                "CDEMPRESA"     => $info['CDEMPRESA'],
                "IDTIPOEMISSAO" => $info['IDTIPOEMISSAO'],
                "NRORG"         => $this->nrorg
            );
            $this->factory->insertParamguiapag($params);
        }
        $this->entityManager->flush();
    }
    
    
    
    #############################################################
    #                                                           #
    #           Funções  de Retorno de Dados                    #
    #                                                           #
    #############################################################
    
    
    
    public function  valoresDefaultPagamento ($cdimposto, $cdempresa, $idtipoemissao) {
        $params = array(
            "CDIMPOSTO"     => $cdimposto=='SN'? null : $cdimposto,
            "CDEMPRESA"     => $cdempresa,
            "IDTIPOEMISSAO" => $idtipoemissao
        ); 
        $resul = $this->factory->getParamguiapag($params);
        return $resul ? [$resul] : array() ;
    }

    /*
        Retorna o Beneficiario. Primeiro verifica se está cadastrado na aba benefiário na tela
        de parâmetros de emissão de guia. Caso esteja vazio verifica o beneficiário cadastrado para 
        o imposto informado.
    */
    public function getBeneficiario ($guia, $parametrosGuia){
        $inscricao     = $guia['IDORGARRECADNFE'] === 'M'? $guia['CDINSCMUNI'] : ($guia['IDORGARRECADNFE'] === 'E' ? $guia['CDINSCESTA']: null );
        $beneficiario  = $this->factory->getBeneficiario($guia['CDEMPRESA'], $inscricao , $guia['CDIMPOSTO'], $guia['IDORGARRECADNFE']);
        if(!$beneficiario){
            if( !$parametrosGuia['CDFORNECED']) {
                throw new \Exception(MessagesFiscal::BENEFICIARIO_VAZIO,1);
            } else {
                $beneficiario = $parametrosGuia['CDFORNECED'];
            }
        } else {
            $beneficiario = $beneficiario['CDFORNECED'];
        }
        
        return $beneficiario;
    }
    
    
    
    #############################################################
    #                                                           #
    #           Funções  de Controle                            #
    #                                                           #
    #############################################################

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|DateTime|string
     */
    public static function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }
    
    /*
     * Funções para gerar a linha digitavel do código de barras.
     */
    public static function geraLinhaDigitavel($codigoBarra) {
        $nrCodigoBarra = str_split($codigoBarra);
        $linhaDigi = self::formataLinhaDigitavel($nrCodigoBarra, '', 1);
        return $linhaDigi;
    }
    
    public static function formataLinhaDigitavel($nrCodigoBarra, $linhaDigi, $index){
        $nrCodigoBarraAux = [];
        $filterCache;
        if($index == 5)
            return str_replace(",", "", $linhaDigi);
        else{
            for($i = 0; $i < 11; $i++){
                $nrCodigoBarraAux[$i] = $nrCodigoBarra[$i];
                unset($nrCodigoBarra[$i]);
            }
            $nrCodigoBarra = array_values($nrCodigoBarra);
            $linhaDigi = $linhaDigi.implode("", $nrCodigoBarraAux).self::calculaDAC($nrCodigoBarraAux);
        }
        return self::formataLinhaDigitavel($nrCodigoBarra, $linhaDigi, $index+1);
    }
    
    public static function multiplicaElementosDAC($nrCodigoBarra){
        $fatorMult = 2;
        for($i=count($nrCodigoBarra)-1; $i >= 0; $i--){
            $nrCodigoBarra[$i] = $nrCodigoBarra[$i] * $fatorMult;
            ($fatorMult == 9) ? $fatorMult = 2 : $fatorMult++;
        }
        return self::somaElementosDAC($nrCodigoBarra);
    }

    public static function somaElementosDAC($nrCodigoBarra){
        $sumtot = 0;
        for($i=0; $i < count($nrCodigoBarra); $i++)
            $sumtot = $sumtot + $nrCodigoBarra[$i];
        return $sumtot;
    }

    public static function calculaDAC($nrCodigoBarra){
        $somaVetor = self::multiplicaElementosDAC($nrCodigoBarra);
        
        if($somaVetor % 11 == 0 || $somaVetor % 11 == 1 )
            return 0;
        else if($somaVetor % 11 == 10)
            return 1;
        else
            return (11 - ($somaVetor % 11));
    }
    
    #############################################################
    #                                                           #
    #           Funções  do Financeiro                          #
    #                                                           #
    #############################################################
    
    public function geraFinanceiro ($guia , $dadosPagameto, $parametrosGuia, $parametrosFiscal, $info){
        $movfluxo = $this->atualizaFluxoCaixa($this->nrorg, $guia['CDEMPRESA'], $dadosPagameto['CDFILIAL'], $parametrosGuia['CDCLASSFINA'], $guia['DTVENCIMENTO'], round($guia['VRTOTAL'],2), 'E', 'S' );

        $this->entityManager->persist($movfluxo);
        $this->entityManager->flush();
        
        $inscricao      = $this->factory->getInscricaoFilial($guia['CDEMPRESA']);
        //Verificar futuramente a parametrização do simples nacional;
        $beneficiario   = $this->getBeneficiario($info, $parametrosGuia);
        $idtpijurnf     = substr($beneficiario,0,1);
        $nrinsjurnf     = substr($beneficiario,1);
        
        $nrlanctonf     = $this->newCode->getCodeByTableName('LANCNF/FILIAL'.$this->environment->getCdFilial(), $this->nrorg);
        $prxnrnotafisc  = $this->newCode->getCodeByTableName('NRNOTAFISCPAG',                            $this->nrorg);
        $prxboleto      = $this->newCode->getCodeByTableName('BOLETPAGAR'.$this->environment->getCdFilial(),    $this->nrorg);
        
        $this->factory->insereNotaFiscal($this->nrorg, $this->environment->getCdFilial(), $dadosPagameto['CDFILIAL'], $nrlanctonf, $guia['CDEMPRESA'], $inscricao['CDINSCESTA'], $inscricao['CDINSCMUNI'], $beneficiario, round($guia['VRTOTAL'],2), $prxnrnotafisc, self::formataData($guia['DTFIMPERIODO']), $idtpijurnf, $nrinsjurnf, $parametrosFiscal['CDTIPOOPERPREPAG'] );
        $this->entityManager->flush();
        
        $this->salvarEdicaoGuia(array_merge($guia, array('NRLANCTONFPAG' =>$nrlanctonf, 'CDFILIALPAG' => $this->environment->getCdFilial()  )));
        $this->entityManager->flush();
        
        
        $this->factory->insereBoleto ($this->nrorg, $this->environment->getCdFilial(), $prxboleto, $beneficiario, $guia['CDEMPRESA'],round($guia['VRTOTAL'],2), self::formataData($guia['DTVENCIMENTO']), $parametrosGuia['CDTPBAIXAPAG'], $guia['CDRECEITA'], $guia['DTFIMPERIODO']);
        $this->entityManager->flush();
        
        $this->factory->insereContaPagar ($this->nrorg, $this->environment->getCdFilial(), $nrlanctonf, self::formataData($guia['DTVENCIMENTO']), round($guia['VRTOTAL'],2), $this->descricaoConta($guia, $info), $prxboleto, $guia['CDEMPRESA'], $parametrosGuia['CDCLASSFINA'], $parametrosGuia['CDTPCTPAGAR'] );
        $this->entityManager->flush();
        
        $this->verificaRateio($dadosPagameto, $nrlanctonf, round($guia['VRTOTAL'],2));

    }
    
    public function geraFinanceiroPorLote ($guias , $dadosPagameto, $parametrosGuia, $parametrosFiscal, $info){
        
        $inscricao      = $this->factory->getInscricaoFilial($info['CDEMPRESA']);
        //Verificar futuramente a parametrização do simples nacional;
        $beneficiario   = $this->getBeneficiario($info, $parametrosGuia);
        $idtpijurnf     = substr($beneficiario,0,1);
        $nrinsjurnf     = substr($beneficiario,1);
        
        $nrlanctonf     = $this->newCode->getCodeByTableName('LANCNF/FILIAL'.$this->environment->getCdFilial(), $this->nrorg);
        $prxnrnotafisc  = $this->newCode->getCodeByTableName('NRNOTAFISCPAG',                            $this->nrorg);
        $prxboleto      = $this->newCode->getCodeByTableName('BOLETPAGAR'.$this->environment->getCdFilial(),    $this->nrorg);
        
        $this->factory->insereNotaFiscal($this->nrorg, $this->environment->getCdFilial(), $dadosPagameto['CDFILIAL'], $nrlanctonf, $info['CDEMPRESA'], $inscricao['CDINSCESTA'], $inscricao['CDINSCMUNI'], $beneficiario, $info['VRTOTAL'], $prxnrnotafisc, self::formataData($info['DTFIMPERIODO']), $idtpijurnf, $nrinsjurnf, $parametrosFiscal['CDTIPOOPERPREPAG'] );
        $this->entityManager->flush();
        
        $this->factory->insereBoleto ($this->nrorg, $this->environment->getCdFilial(), $prxboleto, $beneficiario, $info['CDEMPRESA'], $info['VRTOTAL'], self::formataData($info['DTLIMITEPAG']), $parametrosGuia['CDTPBAIXAPAG'], null, $info['DTFIMPERIODO']);
        $this->entityManager->flush();
        
        $this->factory->insereContaPagar ($this->nrorg, $this->environment->getCdFilial(), $nrlanctonf, self::formataData($info['DTLIMITEPAG']), $info['VRTOTAL'], $this->descricaoConta($guias[0], $info), $prxboleto, $info['CDEMPRESA'], $parametrosGuia['CDCLASSFINA'], $parametrosGuia['CDTPCTPAGAR'] );
        $this->entityManager->flush();
        
        $this->verificaRateio($dadosPagameto, $nrlanctonf, $info['VRTOTAL']);
        
        foreach ($guias as $guia) {
            $movfluxo = $this->atualizaFluxoCaixa($this->nrorg, $info['CDEMPRESA'], $dadosPagameto['CDFILIAL'], $parametrosGuia['CDCLASSFINA'], $guia['DTVENCIMENTO'], round($guia['VRTOTAL'],2), 'E', 'S' );
    
            $this->entityManager->persist($movfluxo);
            $this->entityManager->flush();
            
            $this->salvarEdicaoGuia(array_merge($guia, array('NRLANCTONFPAG' =>$nrlanctonf, 'CDFILIALPAG' => $this->environment->getCdFilial()  )));
            $this->entityManager->flush();
            
        }

    }
    
    public function atualizaFluxoCaixa($nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $vrmovfluxo, $idmovfluxo, $idoperacao){
        try {
            
            // Verifica Identificador Movimentação de Fluxo
            if (! in_array($idmovfluxo,['R','E','P'])){
                throw new \Exception('Identificador de Movimento de Fluxo inválido.');
            }
            $dtmovfluxo = self::formataData($dtmovfluxo);
            

            // Busca Movimentação de Fluxo
            $movfluxo = $this->factory->buscaMovFluxo(
                $nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo
            );
            
            // Verifica Existência Movimentação de Fluxo
            if ($movfluxo) {
                // Atualiza Movimentação de Fluxo
                $movfluxo = $this->factory->atualizaMovFluxo(
                    $movfluxo, $vrmovfluxo, $idoperacao
                );
            } else {

                // Insere Movimentação de Fluxo
                $movfluxo = $this->factory->insereMovFluxo(
                    $nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo, $vrmovfluxo, $idoperacao
                );
            }

            return $movfluxo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }
    
    public function verificaRateio ($dadosPagameto, $nrlanctonf, $vrparcela){
        if(isset ($dadosPagameto['CDRATEIO']) && $dadosPagameto['CDRATEIO']){
            $itensRateio = $this->factory->getItensRateio($dadosPagameto['CDRATEIO']);
            $valorTotal       = 0;
            $maiorValor       = 0;
            $centroCustoMaior ='';
            $filialMaior      ='';
            
            for ($i = 0; $i<count ($itensRateio); $i++ ){
                if(round($vrparcela *$itensRateio[$i]['VRPERATTPCTP'] ,2) > $maiorValor){
                    $centroCustoMaior = $itensRateio[$i]['CDCENTCUST'];
                    $filialMaior      = $itensRateio[$i]['CDFILICCUST'];
                    $maiorValor       = round($vrparcela *$itensRateio[$i]['VRPERATTPCTP'] ,2);
                }
                
                $valorTotal +=   round($vrparcela *$itensRateio[$i]['VRPERATTPCTP'] ,2);
                $this->factory->insereRatctpagar($this->nrorg, $this->environment->getCdFilial(), $itensRateio[$i]['CDFILICCUST'],$nrlanctonf, round($vrparcela *$itensRateio[$i]['VRPERATTPCTP'] ,2), $itensRateio[$i]['CDCENTCUST'], '01');
                $this->entityManager->flush();
            }

            if($valorTotal !=$vrparcela){
                $difereca  = round($vrparcela -$valorTotal,2);
                $this->factory->atualizaDiferencaRateio($filialMaior, $centroCustoMaior, $this->environment->getCdFilial(), $nrlanctonf, $difereca);
                $this->entityManager->flush();
            }
            
        }else{
            $centroCusto = $this->factory->getCentrodeCusto($dadosPagameto['CDFILIAL'])['CDCENTCUST'];
            if($centroCusto){
                $this->factory->insereRatctpagar($this->nrorg, $this->environment->getCdFilial(), $dadosPagameto['CDFILIAL'],$nrlanctonf, $vrparcela, $centroCusto,'01');
                $this->entityManager->flush();
            }
        }
    }
   
    public function descricaoConta ($guia, $info){
        $descricao = '';
        $inscricao = $info['IDORGARRECADNFE'] ==='F'|| $info['IDORGARRECADNFE']=== null ? '':( $info['IDORGARRECADNFE'] ==='E'?' Inscrição Estadual:'.$info['CDINSCESTA']: ' Inscrição Municipal:'.$info['CDINSCMUNI']);
        switch ($guia['IDTIPOGUIA']) {
            case 'G':
                $descricao = $descricao.'Guia Complementar:'.$info['DSIMPOSTONFE'].' Empresa:'.$info['CDEMPRESA']. ' Período:'.self::formataData($guia['DTINIPERIODO'])->format("d/m/Y").'-'.self::formataData($guia['DTFIMPERIODO'])->format("d/m/Y");
                $descricao .= $inscricao.' '.$guia['DESCRICAO']. ' '. $guia['NMMAPAAPUR'];
                break;
            case 'M':
                $descricao = $descricao.'Apuração:'.$info['DSIMPOSTONFE'].' Empresa:'.$info['CDEMPRESA']. ' Período:'.self::formataData($guia['DTINIPERIODO'])->format("d/m/Y").'-'.self::formataData($guia['DTFIMPERIODO'])->format("d/m/Y");
                $descricao .= $inscricao.' '.$guia['NMMAPAAPUR']. ' '. $guia['DESCRICAO'];
                break;
            case 'D':
                break;
            case 'S':
                $descricao = $descricao.'Apuração:'.$info['DSIMPOSTONFE'].' Empresa:'.$info['CDEMPRESA'];
                break;
        }
       
        return substr($descricao,0,200);
    }
    
   
   
    #############################################################
    #                                                           #
    #           Funções de Emissão de Guia                      #
    #                                                           #
    #############################################################
    
    public function emitirGuia ($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $relatorio = array('resultado'=>false, 'relatorio'=>"");
            switch ($row['IDORGARRECADNFE']) {
                case 'F':
                    $relatorio['relatorio'] = $this->emitirGuiaFederal($row);
                    $relatorio['resultado'] = true;
                    break;
                case 'E':
                    switch ($row['SGESTADO']) {
                        case 'ES':
                            $relatorio = $this->emitirGuiaEspiritoSanto($row);
                            break;
                    }
                    break;
                case 'M':
                    
                    break;
            }  
           
            $connection->commit();
            return $relatorio;

        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function emitirGuiaFederal ($row){
        if(DateUtil::getDataAtual() > self::formataData($row['DTVENCIMENTO']) )
            throw new \Exception(MessagesFiscal::GUIA_VENCIDA_CONF, 1);
        
        $empresa = $this->factory->getEmpresa($row['CDEMPRESA']);
        
        $dtAux = isset($row['DTINIPERIODO']) ? $row['DTINIPERIODO'] : $row['DTENTRSAID_DOC'];
        
        $content = http_build_query(
            array(
                "COTAS"=>"",
                "CD_MUNICIPIO_INFORMADA"=>$empresa['CDMUNICIPIO'],
                "Municipio_Classificacao"=>"",
                "UA_Classificacao"=>"",
                "Exercicio"=>"",
                "js"=>"s",
                "TipoBrowser"=>"Darfns.asp",
                "TipTributoReceita"=>"2",
                "DataDoServidor"=> DateUtil::getDataAtual()->format("d/m/Y"),
                "NI_CONTRIBUINTE"=>$row['NRINSJUREMPR'],
                "NU_QUOTA"=>"0000000",
                "UF_SIGLA_INFORMADA"=>$empresa['SGESTADO'],
                "CD_TIPO_DATA_VENCIMENTO"=>"1",
                "CD_TIPO_CALCULO"=>"1",
                "CD_RECEITA"=>$row['CDRECEITA'],
                "CD_TRIBUTO"=>"",
                //"CD_TRIBUTO"=>"0359",
                "IN_TIPO_PERIODO"=>"ME",
                "DatPgtTex"=>DateUtil::getDataAtual()->format("d/m/Y"),
                "MultaIsolada"=>"0",
                "LancOficio"=>"0",
                "TipoDarf"=>"1",
                "CNPJCPF"=>"2",
                "DT_VENCIMENTO_IMPOSTO"=>self::formataData($row['DTVENCIMENTO'])->format("d/m/Y"),
                "NU_REFERENCIA"=>"",
                "DatPgtAge"=>DateUtil::getDataAtual()->format("d/m/Y"),
                "Secao"=>"",
                "Vara"=>"",
                "AcaoClasse"=>"",
                "Reu"=>"",
                "Aliquota"=>"",
                "BaseCalculo"=>"",
                "DataHoraSubmissao"=>$row['DataHoraSubmissao'],
                "Define_Atualizacao_Bancos"=>"S",
                "ANO_PERIODO"=>DateUtil::getAno(self::formataData($dtAux)),
                "MES_PERIODO"=> DateUtil::getMes(self::formataData($dtAux)),
                "DIA_PERIODO"=>"00",
                "DT_VENCIMENTO_MULTA"=>"",
                "DT_PRORROGACAO"=>"",
                "VA_CT_PRINCIPAL"=> number_format($row['VRGUIA'], 2, ',', '.'),
                "CD_MOEDA"=>"6",
                "PE_MULTA_OFICIO"=>"0",
                "CD_IDENTIFICACAO_CT"=>"1",
                "DT_LIMITE_PAGAMENTO"=>self::formataData($row['DTVENCIMENTO'])->format("d/m/Y"),
                "IN_REDUCAO_MULTA"=>"1",
                "VA_IMPOSTO_ATUALIZADO"=>number_format($row['VRTOTAL'], 2, ',', '.'),
                "VA_MULTA_MORA_ATUALIZADO"=>"0",
                "VA_JUROS_MORA_IMPOSTO_ATUALIZADO"=>"0",
                "VA_MULTA_OFICIO_ATUALIZADO"=>"0",
                "VA_JUROS_MORA_MULTA_ATUALIZADO"=>"0",
                "VA_SOMATORIO_JUROS_MORA"=>"0",
                "VA_PERCENTUAL_MULTA_MORA"=>"0",
                "VA_PERCENTUAL_JUROS_MORA_IMPOSTO"=>"0",
                "VA_PERCENTUAL_JUROS_MORA_MULTA"=>"0",
                "DT_ATUALIZACAO_MONETARIA"=>DateUtil::getUltimoDiaDoMes(self::formataData($dtAux))->format("d/m/Y"),
                //"DT_ATUALIZACAO_MONETARIA"=>DateUtil::getUltimoDiaDoMes(DateUtil::getDataAtual())->format("d/m/Y"),
                "DT_DATA_CONSOLIDACAO"=>DateUtil::getDataAtual()->format("d/m/Y"),
                "VA_PERCENTUAL_REDUCAO_MULTA"=>"0",
                "VA_TOTAL_DEVIDO"=>number_format($row['VRTOTAL'], 2, ',', '.'),
                "DT_PA_CALCULO"=>DateUtil::getUltimoDiaDoMes(self::formataData($dtAux))->format("d/m/Y"),
                "PagSubmit"=>"0",
                "DT_ALIENACAO"=>"",
                "Num_Princ"=> substr( $row['NRINSJUREMPR'],0,12 ),
                "Num_DV"   => substr( $row['NRINSJUREMPR'],12 ),
                "radioGanho"=>"",
                "Documento"=>"",
                "versao"=>"1.6.66",
                "CD_MUNICIPIO_BASE_CPF"=>"7535",
                "CD_ORGAO_SRF_BASE_CPF"=>"00910100",
                "ProgramaChamador"=>"SicalcWeb versão 1.6.66.4123",
                "UF_SIGLA_BASE_CPF"=>"PR",
                "NO_NOME_CONTRIBUINTE"=>$row['NMEMPRESA']
            )
        );
        $options = array('http' => array(
            'method' => "POST",
            'header' =>
                "Authorization: Bearer sdf541gs6df51gsd1bsb16etb16teg1etr1ge61g\n" . // ---
                "Content-Type: application/x-www-form-urlencoded",
            'content' => $content
        ));
        $context = stream_context_create($options);
        
        $response = file_get_contents('http://www31.receita.fazenda.gov.br/Darf/senda.asp', FALSE, $context);
        
        // Pega o código de barras e a linha diditável gerado para ser utlilizado pelo sistema;
        preg_match_all('/<span class="codigo"(.+)<\/span>/', $response, $conteudo);
            
            
        $linhaDigitavel = '';
        $codigoBarras   = '';
            
        if(!empty($conteudo[0])){
            $codigo         = $conteudo[0][0];
            $linhaDigitavel = str_replace ('-','',str_replace('</span>','',(str_replace('<span class="codigo">','',$codigo))));
            if($linhaDigitavel){
                $codigoBarras   .= substr($linhaDigitavel,0,11);
                $codigoBarras   .= substr($linhaDigitavel,12,11);
                $codigoBarras   .= substr($linhaDigitavel,24,11);
                $codigoBarras   .= substr($linhaDigitavel,36,11);
            }
        }
            
        $boleto = $this->factory->getBoleto($row['CDFILIALPAG'], $row['NRLANCTONFPAG']);
        $this->factory->atualizaBoleto($row['CDFILIALPAG'], $boleto['NRLANBOLPAG'], $linhaDigitavel,$codigoBarras);
        $this->entityManager->flush();
        
        return $response;
    }   
        
    public function getArquivoGuiaGnreTxt( $rows ) {
        $arquivo = array();
        
        //Percorre as rows e concatena os resultados em uma string separados por '\t'
        foreach( $rows as $row ) {
            $linha = $this->factory->getDadosGuiaGnreTxt(
                $this->salvarEdicaoGuia( $row ), //retorna o NRSEQDOCARRECAD
                $this->nrorg );
            
            //Converte a saída em array para uma string com registros separados por tabulação e os adiciona a uma array que será o arquivo
            if( is_array( $linha ) ) {
                $arquivo[] = implode( "\t", $linha );
            }
        }
        
        //Converte e retorna o array de registros (linhas) em uma string separada por \r\n (quebra de linha)
        return implode( "\r\n", $arquivo );
    }
    
    public function emitirGnreOnline($dataset) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $rows = $dataset[0];
            $info = $dataset[1];
            
            $response = [];

            foreach($rows as $row) {
                if(!empty($row['NRSEQLOTE'])) {
                    $this->factory->updateDocarrecadpagNewLote($row['NRSEQLOTE']);
                    $this->entityManager->flush();
                }
            }  
                
            $params = array(
                "CDEMPRESA"     => $info["CDEMPRESA"],
                "CDINSCESTA"     => $info["CDINSCESTA"]
            );
            
            $parametro = $this->factory->getParamtrans($params);
            
            // var_dump($rows, $info, $datasPagamento);
            $xml       = $this->getXmlGnreOnline($rows, $info, $parametro);
            // var_dump($xml);die;
            $this->setCertFileKey($info, $parametro);
            
            $resultado = $this->conectaWebServiceGnreOnline($xml, $parametro['IDAMBIENTE'] === '1' ? self::ENVIAR_LOTE_PROD : self::ENVIAR_LOTE_TEST, "http://www.gnre.pe.gov.br" );
            $retorno   = $this->trataRetornoEnvioLoteGnreOnline($resultado, $rows, $info);
            
            array_push($response, $retorno);
            
            $connection->commit();
            return $response;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    #############################################################
    #                                                           #
    #           Funções de XML(WEB SERVICE GUIAS)               #
    #                                                           #
    #############################################################
    
    public function conectaWebService($xml, $url, $action) {
        try {
            // var_dump($xml);
            $msgSize = strlen($xml);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: \"$action\"",
                "Content-length: $msgSize"
            ];
            
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_VERBOSE, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); 
            curl_setopt($ch, CURLOPT_ENCODING, 'gzip,deflate'); 
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/xml'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $parameters);
            $response = curl_exec($ch);
            curl_close($ch);

            $xmlDom = new \DOMDocument( '1.0', 'utf-8' ); 
            $xmlDom->formatOutput = true;           
            $xmlDom->preserveWhiteSpace = false;  
            $xmlDom->loadXML($response);
           
            return  array("xml"=>$response, "xmlDom"=>$xmlDom);
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function conectaWebServiceGnreOnline($xml, $url, $action) {
        try {
            
            $msgSize = strlen($xml);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: $action",
                "Content-length: $msgSize"
            ];
            
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($ch, CURLOPT_HEADER, 1);
            curl_setopt($ch, CURLOPT_SSLCERT, $this->_certKeyFile);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_VERBOSE, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_ENCODING, 'gzip,deflate'); 
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/xml'));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $parameters);
            
            $response = curl_exec($ch);
            
            // var_dump($response);
            
            $this->soaperror = curl_error($ch);
            $ainfo = curl_getinfo($ch);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
            
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado. ERRO: '.$httpcode, 1);
            }
            
            $xmlDom = new \DOMDocument( '1.0', 'utf-8' ); 
            $xmlDom->formatOutput = true;           
            $xmlDom->preserveWhiteSpace = false;  
            $xmlDom->loadXML($responseBody);
           
            return  array("xml"=>$responseBody, "xmlDom"=>$xmlDom);
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function buildSoapCall($cert, $wsdl, $action, $soapVar, $xmlLote) {
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: $action",
                "Content-length: $msgSize"
            ];
            
            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado. ERRO: '.$httpcode, 1);
            } else {
                $xml = new \SimpleXMLElement($responseBody);
                $domxml = new \DOMDocument();
                $domxml->preserveWhiteSpace = false;
                $domxml->formatOutput = true;
                $domxml->loadXML($xml->asXML());
                $result = $domxml->getElementsByTagName('Reinf')->item(0);
                $reinfXML = $result->ownerDocument->saveXML($result);
                
                $stdClass = new \StdClass;
                $reinf = new \StdClass;
                $reinf->any = $reinfXML;
                $stdClass->ReceberLoteEventosResult = $reinf;
                    
                return $stdClass;
            }
            
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    public function getXmlEmissaoGuiaEspiritoSanto ($rows, $info, $parametro){
        
        /*  Pelo amor de deus, o proximo faz esse trem funcionar <3 - Tiago F.  */
			
        $row = $rows[0];
        $inscricaoEstadual = $this->factory->getInscricaoEstadual($info['CDEMPRESA'], $info['CDINSCESTA']);
        $info['CDIMPOSTO'] = '80';
        $completeInfo = $this->factory->getComplementInfo(
                                                        $info['SGESTADO'], 
                                                        $row['NRLANCTONF'], 
                                                        $info['CDINSCESTA'], 
                                                        $info['OPCGUIA'], 
                                                        ($info['IDTIPOEMISSAO'] === 'A' ? 'S' : $row['OPERACAO_DOC']), 
                                                        $info['CDEMPRESA']);

        switch ($row['CDRECEITA']) {
            case '100013':
                $codServ = '1244';
                break;
            case '100021':
                $codServ = '1236';
                break;
            case '100030':
                $codServ = '1252';
                break;
            case '100048':
                if ($inscricaoEstadual['SGESTADO'] === 'ES') {
                    $codServ = '1384';
                } else {
                    $codServ = '1376';
                }
                break;
            case '100056':
                $codServ = '2569';
                break;
            case '100099':
                $codServ = '1376';
                break;
            default :
                $codServ = 'null';
        }
        
        /*$xml ='
            <?xml version="1.0" encoding="utf-8"?>
                <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                    <soap12:Header>
                        <duaServiceHeader xmlns="http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao">
                            <versao>string</versao>
                        </duaServiceHeader>
                    </soap12:Header>
                    <soap12:Body>
                        <duaDadosMsg xmlns="http://www.sefaz.es.gov.br/duae">
                            <emisDua>
                                <versao>1.01</versao>
                                <tpAmb>2</tpAmb>
                                <cnpjEmi>'.$row['NRINSJUREMPR'] .'</cnpjEmi>
                                <cnpjOrg></cnpjOrg>
                                <cArea></cArea> 
                                <cServ></cServ> 
                                <cnpjPes></cnpjPes> 
                                <dRef>'.self::formataData($row['DTINIPERIODO'])->format('Y-m').'</dRef> 
                                <dVen>'.self::formataData($row['DTVENCIMENTO'])->format('Y-m-d').'</dVen> 
                                <dPag>'.self::formataData($row['DTVENCIMENTO'])->format('Y-m-d').'</dPag> 
                                <cMun>'.$inscricaoEstadual['CDMUNICIPIO'].'</cMun> 
                                <xInf></xInf> 
                                <vMul>0</vMul> 
                                <vJur>0</vJur> 
                                <vAtu>'.$row['VRGUIA'].'</vAtu> 
                                <vRec>'.$row['VRGUIA'].'</vRec> 
                                <qtde></qtde> 
                                <xIde></xIde>
                            </emisDua>
                         </duaeDadosMsg>
                    </soap12:Body>
                </soap12:Envelope>';
                
                
                
                
        */       
        // var_dump($parametro);die;
        $this->setCertFileKey($info, $parametro);
            
            /*$xmlArea ='
            <?xml version="1.0" encoding="utf-8"?>
                <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                    <soap12:Header>
                        <duaServiceHeader xmlns="http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao">
                            <versao>string</versao>
                        </duaServiceHeader>
                    </soap12:Header>
                    <soap12:Body>
                        <duaDadosMsg xmlns="http://www.sefaz.es.gov.br/duae">
                            <consAreaServico>
                                <versao>1.1</versao>
                                <tpAmb>2</tpAmb>
                                <cnpj>'.$inscricaoEstadual['NRINSJURIEST'] .'</cnpjEmi>
                            </consAreaServico>
                         </duaeDadosMsg>
                    </soap12:Body>
                </soap12:Envelope>';*/
                
        $xmlArea ='<?xml version="1.0" encoding="utf-8"?>
            <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                <soap12:Header>
                    <duaServiceHeader xmlns="http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao">
                        <versao>string</versao>
                    </duaServiceHeader>
                </soap12:Header>
                <soap12:Body>
                <duaDadosMsg xmlns="http://www.sefaz.es.gov.br/duae">';
                
        $xmlArea .='<emisDua>
            <versao>1.01</versao>
                    <tpAmb>'.$parametro['IDAMBIENTE'].'</tpAmb>
                    <cnpjEmi>'.$completeInfo['CNPJ'].'</cnpjEmi>
                    <cnpjOrg>27.080.571/0001-30</cnpjOrg>
                    <cArea>ICMS</cArea>';
                if($codServ) $xmlArea .='<cServ>'.$codServ.'</cServ>';
                $xmlArea .= '<cnpjPes>'.$inscricaoEstadual['NRINSJURIEST'].'</cnpjPes>
                    <dRef>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).
                    '-'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</dRef>
                    <dVen>'.$this->formataDataGnre($row['DTVENCIMENTO']).'</dVen>
                    <dPag>'.$this->formataDataGnre($row['DTLIMITEPAG']).'</dPag>
                    <cMun>'.substr($inscricaoEstadual['CDMUNICIBGE'], 2).'</cMun>';
                    // <xInf>'..'</xInf>
        if($row['VRMULTA']) $xmlArea .='<vMul>'.$row['VRMULTA'].'</vMul>';
        if($row['VRJUROS'])  $xmlArea .='<vJur>'.$row['VRJUROS'].'</vJur>';
        $xmlArea .='<vAtu>'.$row['VRGUIA'].'</vAtu>
                    <vRec>'.$row['VRTOTAL'].'</vRec>';
        if($codServ) $xmlArea .='<qtde>'.$codServ.'</qtde>';
        $xmlArea .='<xIde>1</xIde>
                </emisDua>';
        $xmlArea .='</duaDadosMsg>
                    </soap12:Body>
                </soap12:Envelope>';
        
        
        $msgSize = strlen($xmlArea);
        $parameters = [
            "Content-Type: text/xml;charset=UTF-8",
            //"SOAPAction: \"http://www.sefaz.es.gov.br/duae/wsdl/duaEmissao\"",
            "Content-length: $msgSize"
        ];
        // http://app.sefaz.es.gov.br/DuaPortal
        //duaEmissao
        // http://www.sefaz.es.gov.br/duae/wsdl/duaEmissao
        // http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao
        
        // var_dump($this->_certKeyFile);
        
        $oCurl = curl_init();
        curl_setopt($oCurl, CURLOPT_URL, $parametro['IDAMBIENTE'] === 1 ? self::ENVIAR_ES_LOTE_PROD : self::ENVIAR_ES_LOTE_TEST );
        
        //  https://app.sefaz.es.gov.br/WsDua
        // curl_setopt($oCurl, CURLOPT_URL, "https://app.sefaz.es.gov.br/WsDua");
        
        curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        curl_setopt($oCurl, CURLOPT_HEADER, 1);
        
        // curl_setopt($oCurl, CURLOPT_HTTP_VERSION, 'CURL_HTTP_VERSION_2_0');
        
        curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
        curl_setopt($oCurl, CURLOPT_SSLCERT, $this->_certKeyFile);
        curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($oCurl, CURLOPT_VERBOSE, 1);
        curl_setopt($oCurl, CURLOPT_POST, false);
        curl_setopt($oCurl, CURLOPT_ENCODING, 'gzip,deflate'); 
        curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlArea);
        curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
        curl_setopt($oCurl, CURLOPT_HTTPGET, TRUE);
    
        // $response = curl_exec($oCurl);
        
        
        $response = curl_exec($oCurl);
        $this->soaperror = curl_error($oCurl);
        $ainfo = curl_getinfo($oCurl);
        if (is_array($ainfo)) {
            $this->soapinfo = $ainfo;
        }
        $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
        $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
        curl_close($oCurl);

        // var_dump(curl_getinfo($oCurl));
    
        return $response;
    }
    
    public function getXmlTransmissaoLoteRj($rows, $info){
        
        //Tipo de Documento 1 DARJ | 2 - GNRE
        $inscricaoEstadual = $this->factory->getInscricaoEstadual($info['CDEMPRESA'], $info['CDINSCESTA']);
        $empresa           = $this->factory->getEmpresa($info['CDEMPRESA']);
        $endereco          = $inscricaoEstadual['DSENDEIEST'].', '.($inscricaoEstadual['DSCOMPENDIES']?$inscricaoEstadual['DSCOMPENDIES'].', ' :' ').($inscricaoEstadual['NMBAIRIEST']?$inscricaoEstadual['NMBAIRIEST']:'') ;
        
        
        $xml ='<?xml version="1.0" encoding="UTF-8"?>
                <soapenv:Envelope
                    xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
                    <soapenv:Header/>
                    <soapenv:Body>
                        <enviarDados
                            xmlns="http://www.openuri.org/">
                            <emitente>
                                <CnpjEmitente>'.$empresa['NRINSJUREMPR'].'</CnpjEmitente>
                                <Email>'.$empresa['DSEMAILEMPRESA'].'</Email>
                            </emitente>
                            <documentos>
                                <Documento>
                                    <TipoPagamento>1</TipoPagamento>
                                    <TipoDocumento>'.$info['TipoDocumento'].'</TipoDocumento>
                                    <DataPagamento>'.self::formataData($rows[0]['DTLIMITEPAG'])->format("d/m/Y").'</DataPagamento>
                                    <SqDocumento>0</SqDocumento>
                                    <ItensPagamentos>';
                
                foreach ($rows as $row) {                        
                    $completeInfo = $this->factory->getComplementInfo(
                                                        $info['SGESTADO'], 
                                                        $row['NRLANCTONF'], 
                                                        $info['CDINSCESTA'], 
                                                        ($info['TipoDocumento'] == 'G' ? 'G' : 'D'), 
                                                        ($info['IDTIPOEMISSAO'] === 'A' ? 'S' : $row['OPERACAO_DOC']), 
                                                        $info['CDEMPRESA']
                                                    );
                    $xml .= '
                                        <ItemPagamento>
                                            <TipoId>1</TipoId>
                                            <Cnpj>'.$empresa['NRINSJUREMPR'].'</Cnpj>
                                            <DataVencimento>'.self::formataData($row['DTVENCIMENTO'])->format("d/m/Y").'</DataVencimento>
                                            <ValorICMSPrincipal>'.(float)$row['VRGUIA'].'</ValorICMSPrincipal>
                                            <ValorFECPPrincipal>0</ValorFECPPrincipal>
                                            <ValorTotal>'.(float)$row['VRTOTAL'].'</ValorTotal>
                                            <NomeRazaoSocial>'.$empresa['NMRAZSOCEMPR'].'</NomeRazaoSocial>
                                            <NumControleContribuinte>'.$row['NRSEQDOCARRECAD'].'</NumControleContribuinte>
                                            <InscEstadualRJ>'.$inscricaoEstadual['CDINSCESTA'].'</InscEstadualRJ>
                                            <EnderecoContribuinte>'.substr($endereco,0,70).'</EnderecoContribuinte>
                                            ';
                        
            if($inscricaoEstadual['NRDDDTELEIEST'] && $inscricaoEstadual['NRTELEIESTSEP'])
               $xml .= '                    <DddContribuinte>'. $inscricaoEstadual['NRDDDTELEIEST'].'</DddContribuinte>
                                            <TelefoneContribuinte>'. $inscricaoEstadual['NRTELEIESTSEP'].'</TelefoneContribuinte>
                                            ';
                            
            $xml .='                        <CepContribuinte>'.$inscricaoEstadual['NRCEPIEST'].'</CepContribuinte>
                                            <MunicipioContribuinte>'.$inscricaoEstadual['NMMUNICIPIO'].'</MunicipioContribuinte>
                                            <UfContribuinte>'.$inscricaoEstadual['SGESTADO'].'</UfContribuinte>
                                            <DiaVencimento>'.DateUtil::getDia(self::formataData($row['DTVENCIMENTO'])).'</DiaVencimento>
                                            <Natureza>'.$row['CDNATUREZA'].'</Natureza>
                                            ';
                        
            switch($row['CDNATUREZA']) {
                case '1':
                    $xml.= '                <NaturezaQualificacao>1</NaturezaQualificacao>
                                            <PeriodoReferenciaMes>'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaMes>
                                            <PeriodoReferenciaAno>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaAno>
                                            ';
                    
                    break;
                case '2': 
                    if ( !empty($completeInfo['PRODUTO'])) $xml.= '
                                            <CodigoProduto>'.$completeInfo['PRODUTO'].'</CodigoProduto>
                                            ';//     := adqGeraGuiaRJPRODUTO.AsString;
              $xml.= '<PeriodoReferenciaMes>'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaMes>
                      <PeriodoReferenciaAno>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaAno>
                      ';
                    
                    break;
                case '3':
                    $xml.= '<PeriodoReferenciaMes>'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaMes>
                            <PeriodoReferenciaAno>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).'</PeriodoReferenciaAno>
                            ';
                    
                    break;
                case '4': 
                    $xml.= '<TipoApuracao>'.$info['TipoApuracao'] .'</TipoApuracao>';
                    if (!empty($completeInfo['PRODUTO'])) $xml.= '<CodigoProduto>'.$completeInfo['PRODUTO'].'</CodigoProduto>';//     := adqGeraGuiaRJPRODUTO.AsString;
                    
                    if($info['TipoApuracao'] == '2') {
                        if(!empty($completeInfo['CNPJ'])) { 
                            $xml.= '<NotaFiscalCnpj>'.$completeInfo['CNPJ'].'</NotaFiscalCnpj>
                            ';
                        } else {
                            $xml.= '<NotaFiscalCpf>'.$completeInfo['CPF'].'</NotaFiscalCpf>
                            ';                            
                        }
                        $xml.= '<NotaFiscalDataEmissao>'.$completeInfo['DTEMISSAO'].'</NotaFiscalDataEmissao>
                                <NotaFiscalNumero>'.$completeInfo['NUMERO_NF'].'</NotaFiscalNumero>
                                <NotaFiscalSerie>'.$completeInfo['SERIE'].'</NotaFiscalSerie>
                                <NotaFiscalTipo>'.$completeInfo['TIPO_NF'].'</NotaFiscalTipo>
                                ';
                    } else if ($info['TipoApuracao'] == '1') {
                        $dtInicialPeriodo = intval(DateUtil::getDia(self::formataData($row['DTINIPERIODO'])));
                        
                        if( $dtInicialPeriodo > 0 && $dtInicialPeriodo < 11) {
                            $completeInfo['PERIODO_REF_DECENDIO'] = 1;
                        } else if( $dtInicialPeriodo > 10 && $dtInicialPeriodo < 21) {
                            $completeInfo['PERIODO_REF_DECENDIO'] = 2;
                        } else if( $dtInicialPeriodo > 20) {
                            $completeInfo['PERIODO_REF_DECENDIO'] = 3;
                        } else {
                            $completeInfo['PERIODO_REF_DECENDIO'] = '';
                        }
                        
                        if( $dtInicialPeriodo === 1) {
                            $completeInfo['TIPO_PERIODO_APUR'] = 'M';
                        } else {
                            $completeInfo['TIPO_PERIODO_APUR'] = 'D';
                        }
                        
                        $xml.= '
                                            <TipoPeriodoApuracao>'.$completeInfo['TIPO_PERIODO_APUR'].'</TipoPeriodoApuracao>
                                            <PeriodoReferenciaMes>'.DateUtil::getMes(self::formataData($row['DTINIPERIODO'])).'</PeriodoReferenciaMes>
                                            <PeriodoReferenciaAno>'.DateUtil::getAno(self::formataData($row['DTINIPERIODO'])).'</PeriodoReferenciaAno>
                                            <PeriodoReferenciaDecendio>'.$completeInfo['PERIODO_REF_DECENDIO'].'</PeriodoReferenciaDecendio>
                                            ';
                    }
                    
                    break;
                case '5':
                    // if(isset($row['TIPO_IMPORT'])) $xml.= '<TipoImportacao>'.$row['TIPO_IMPORT'].'</TipoImportacao>';
                    // if(isset($row['DOC_ORIGEM'])) $xml.= '<NumDocOrigem>'.$row['DOC_ORIGEM'].'</NumDocOrigem>';
                    $xml.= '<TipoImportacao>1</TipoImportacao>';
                    $xml.= '<NumDocOrigem></NumDocOrigem>
                    ';
                    
                    break;
                case '6': 
                    $xml.='<PeriodoReferenciaMes>'.DateUtil::getMes(self::formataData($row['DTINIPERIODO'])).'</PeriodoReferenciaMes>
                          <PeriodoReferenciaAno>'.DateUtil::getAno(self::formataData($row['DTINIPERIODO'])).'</PeriodoReferenciaAno>
                          ';
                    
                    break;
                case '7': 
                    $xml.='<DataFatoGerador>'.$completeInfo['DATA_FATO_GER'].'</DataFatoGerador>
                    ';
                    
                    break;
            }
            
            $xml.='</ItemPagamento>';
        }
            
        $xml.='</ItensPagamentos>
                </Documento>
            </documentos>
        </enviarDados>
        </soapenv:Body>
        </soapenv:Envelope>';
        
        return $xml;
    }
    
    /*public function getXmlTransmissaoLoteEs($rows, $info, $parametro){
        $row = $rows[0];
        $inscricaoEstadual = $this->factory->getInscricaoEstadual($info['CDEMPRESA'], $info['CDINSCESTA']);
        $info['CDIMPOSTO'] = '80';
        $completeInfo = $this->factory->getComplementInfo(
                                                        $info['SGESTADO'], 
                                                        $row['NRLANCTONF'], 
                                                        $info['CDINSCESTA'], 
                                                        $info['OPCGUIA'], 
                                                        ($info['IDTIPOEMISSAO'] === 'A' ? 'S' : $row['OPERACAO_DOC']), 
                                                        $info['CDEMPRESA']);

        
        switch ($row['CDRECEITA']) {
            case '100013':
                $codServ = '1244';
                break;
            case '100021':
                $codServ = '1236';
                break;
            case '100030':
                $codServ = '1252';
                break;
            case '100048':
                if ($inscricaoEstadual['SGESTADO'] === 'ES') {
                    $codServ = '1384';
                } else {
                    $codServ = '1376';
                }
                break;
            case '100056':
                $codServ = '2569';
                break;
            case '100099':
                $codServ = '1376';
                break;
        }
        
        $xml ='<?xml version="1.0" encoding="utf-8"?>
                <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                    <soap12:Header>
                        <duaServiceHeader xmlns="http://www.sefaz.es.gov.br/duae/wsdl/DuaEmissao">
                            <versao>string</versao>
                        </duaServiceHeader>
                    </soap12:Header>
                    <soap12:Body>
                    <duaDadosMsg xmlns="http://www.sefaz.es.gov.br/duae">';
        
        $xml .='<emisDua>
                    <versao>1.01</versao>
                    <tpAmb>'.$parametro['IDAMBIENTE'].'</tpAmb>
                    <cnpjEmi>'.$completeInfo['CNPJ'].'</cnpjEmi>
                    <cnpjOrg>27.080.571/0001-30</cnpjOrg>
                    <cArea>ICMS</cArea>
                    <cServ>'.$codServ.'</cServ>
                    <cnpjPes>'.$inscricaoEstadual['NRINSJURIEST'].'</cnpjPes>
                    <dRef>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).
                    '-'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</dRef>
                    <dVen>'.$this->formataDataGnre($row['DTVENCIMENTO']).'</dVen>
                    <dPag>'.$this->formataDataGnre($row['DTLIMITEPAG']).'</dPag>
                    <cMun>'.substr($inscricaoEstadual['CDMUNICIBGE'], 2).'</cMun>';
                    // <xInf>'..'</xInf>
        if($row['VRMULTA']) $xml .='<vMul>'.$row['VRMULTA'].'</vMul>';
        if($row['VRJUROS'])  $xml .='<vJur>'.$row['VRJUROS'].'</vJur>';
        $xml .='<vAtu>'.$row['VRGUIA'].'</vAtu>
                    <vRec>'.$row['VRTOTAL'].'</vRec>
                    <qtde>'.$codServ.'</qtde>
                    <xIde>1</xIde>
                </emisDua>';
                
                    $xml .='</duaDadosMsg>
                 </soap12:Body>
                </soap12:Envelope> ';
        
        return $xml;
    }*/
    
    public function getXmlConsultaLoteRj($row){
        $lote    =  $this->factory->getLotegnre($row['NRSEQLOTE']);
        $empresa = $this->factory->getEmpresa($lote['CDEMPRESA']);

        $xml ='<?xml version="1.0" encoding="UTF-8"?>
                <soapenv:Envelope
                    xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
                    <soapenv:Header/>
                    <soapenv:Body>
                        <consultarDados xmlns="http://www.openuri.org/">>
                            <cnpj>'.$empresa['NRINSJUREMPR'].'</cnpj>
                            <idSessao>'.$lote['NRRECIBO'].'</idSessao>
                        </consultarDados>
                    </soapenv:Body>
                </soapenv:Envelope>';
        return $xml;
    }
    
    public function getXmlGnreOnline($rows, $info, $parametro){
        $inscricaoEstadual = $this->factory->getInscricaoEstadual('15', '2321425390063');
        $empresa           = $this->factory->getEmpresa('15');
        // $inscricaoEstadual = $this->factory->getInscricaoEstadual($info['CDEMPRESA'], $info['CDINSCESTA']);
        // $empresa           = $this->factory->getEmpresa($info['CDEMPRESA']);
        $endereco          = $inscricaoEstadual['DSENDEIEST'].', '.($inscricaoEstadual['DSCOMPENDIES']?$inscricaoEstadual['DSCOMPENDIES'].', ' :' ').($inscricaoEstadual['NMBAIRIEST']?$inscricaoEstadual['NMBAIRIEST']:'') ;
        
        $xml = '<?xml version="1.0" encoding="UTF-8"?>
                <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                <soap12:Header>
                    <gnreCabecMsg xmlns="http://www.gnre.pe.gov.br/wsdl/processar">
                        <versaoDados>2.00</versaoDados>
                    </gnreCabecMsg>
                </soap12:Header>
                <soap12:Body>
                <gnreDadosMsg xmlns="http://www.gnre.pe.gov.br/webservice/GnreLoteRecepcao">
                <TLote_GNRE versao="2.00" xmlns="http://www.gnre.pe.gov.br">
                <guias>';
                
                foreach($rows as $row) {
                    
                    $configUf          = $this->factory->getConfigUf($info['UFFAVORECIDO'], $row['CDRECEITA']);
                    $completaInfo      = $this->factory->completaInfoGnreOnline($info['CDIMPOSTO'], $info['CDEMPRESA'], $info['SGESTADO'], $row['NRLANCTONF'], $row['CDFILIAL']);
                    
                    $xml.= '<TDadosGNRE versao="2.00">
                      <ufFavorecida>'.$info['UFFAVORECIDO'].'</ufFavorecida>'
                        // 	Contém código do tipo da GNRE.
                        // 0 - Guia Simples
                        // 1 - Guia com Múltiplos Documentos de Origem
                        // 2 - Guia com Múltiplas Receitas
                      .'<tipoGnre>'.'0'.'</tipoGnre>';
                    
                    if($configUf['IDEMITENTE'] === 'S') {
                        $xml.= '<contribuinteEmitente>
                           <identificacao>';
                        if($info['SGESTADO'] == $info['UFFAVORECIDO']) {
                            $xml.= '<IE>'.$inscricaoEstadual['CDINSCESTA'].'</IE></identificacao></contribuinteEmitente>';
                        } else {
                           if(!empty($inscricaoEstadual['NRINSJURIEST'])) {
                                $xml.= '<CNPJ>'.$inscricaoEstadual['NRINSJURIEST'].'</CNPJ></identificacao>';    
                            } else {
                                $xml.= '<CPF>'.$inscricaoEstadual['CPFEMIT'].'</CPF></identificacao>';    
                            }  
                            $xml.= '<razaoSocial>'.$empresa['NMRAZSOCEMPR'].'</razaoSocial>
                                   <endereco>'.$inscricaoEstadual['DSENDEIEST'].'</endereco>
                                   <municipio>'.substr($inscricaoEstadual['CDMUNICIBGE'], 2).'</municipio>
                                   <uf>'.$inscricaoEstadual['SGESTADO'].'</uf>
                                   <cep>'.$inscricaoEstadual['NRCEPIEST'].'</cep>
                                   <telefone>'.$inscricaoEstadual['NRTELEIEST'].'</telefone>
                                  </contribuinteEmitente>';
                        }
                    }
                    $xml.= '<itensGNRE>
                                <item>
                                <receita>'.$row['CDRECEITA'].'</receita>';
                        if($configUf['IDDETRECEITA'] === 'S') {
                            $xml.='<detalhamentoReceita>'.$row['CDDETRECEITA'].'</detalhamentoReceita>';
                        }
                        
                        if($configUf['IDDOCORIGEM'] === 'S') {
                            if($row['CDRECEITA'] === '100056') {
                                $xml.='<documentoOrigem tipo="04" >'.$completaInfo['NRNOTAFISC'].'</documentoOrigem>';
                            } else {
                                $xml.='<documentoOrigem tipo="'.$completaInfo['TPDOCUMENTO'].'" >'.$completaInfo['NRNOTAFISC'].'</documentoOrigem>';
                            }
                        }
                        
                        if($configUf['IDPRODUTO'] === 'S') {
                            $xml.='<produto>'.$completaInfo['PRODUTO'].'</produto>';
                        }
                        
                        if($configUf['IDPERIODOAPUR'] === 'S') {
                            $xml.='<referencia>
                             <periodo>'.'0'.'</periodo>';
                            if($configUf['IDPERIODOREF'] === 'S') {
                                 $xml.='<mes>'.DateUtil::getMes(self::formataData($row['DTVENCIMENTO'])).'</mes>
                                 <ano>'.DateUtil::getAno(self::formataData($row['DTVENCIMENTO'])).'</ano>
                                </referencia>';
                            }
                        }
                        
                        
                        // if($configUf['IDDTVENCIMENTO'] === 'S') {
                        $xml.='<dataVencimento>'.$this->formataDataGnre($row['DTVENCIMENTO']).'</dataVencimento>';
                        // }
                        
                        if($configUf['IDVRPRINCIPAL'] === 'S') {
                            $xml.='<valor tipo="11" >'.number_format((float)$row['VRGUIA'], 2, '.', '').'</valor>';
                        }
                        
                        if($configUf['IDVRTOTAL'] === 'S') {
                            $xml.='<valor tipo="21" >'.number_format((float)$row['VRTOTAL'], 2, '.', '').'</valor>';
                        }
                        
                        if($configUf['IDCONVENIO'] === 'S') {
                            if($completaInfo['CONVENIO']) $xml.='<convenio>'.$completaInfo['CONVENIO'].'</convenio>';
                        }
                        
                        if($configUf['IDDESTINATARIO'] === 'S') {
                            $xml.='<contribuinteDestinatario>
                             <identificacao>';
                             if(empty($completaInfo['INSCESTADEST'])){
                                 $xml.='<IE>'.$completaInfo['INSCESTADEST'].'</IE>';
                             } else if($row['IDTPDESTEMIT_DOC'] === 'C') {
                                 $xml.='<CPF>'.$row['NRIJDESTEMIT_DOC'].'</CPF>';
                             } else if($row['IDTPDESTEMIT_DOC'] === 'J') {
                                  $xml.='<CNPJ>'.$row['NRIJDESTEMIT_DOC'].'</CNPJ>';
                             }
                             $xml.='</identificacao>
                                 <razaoSocial>'.$row['NMDESTEMIT_DOC'].'</razaoSocial>
                                 <municipio>'.$completaInfo['MUNICIPIODEST.'].'</municipio>
                                </contribuinteDestinatario>';
                        }
                        
                        if($configUf['IDCAMPOEXTRA'] === 'S') {
                            $adqCampoExtraObrig = $this->factory->adqCampoExtraObrig($info['UFFAVORECIDO'], $row['CDRECEITA']);
                            
                            if($adqCampoExtraObrig['NRCODIGO']) {
                                $xml.='<camposExtras>
                                        <campoExtra>
                                          <codigo>'.$adqCampoExtraObrig['NRCODIGO'].'</codigo>
                                          <valor>'.$completaInfo['CHAVE'].'</valor>
                                         </campoExtra>
                                      </camposExtras>';
                            }
                        }
                        
                       $xml.='</item>
                      </itensGNRE>';
                       
                        $xml.='<valorGNRE>'.number_format((float)$row['VRTOTAL'], 2, '.', '').'</valorGNRE>';
                      
                        $xml.='<dataPagamento>'.$this->formataDataGnre($row['DTLIMITEPAG']).'</dataPagamento>';
                        
                        $xml.='<identificadorGuia>'.$row['NRSEQDOCARRECAD'].'</identificadorGuia>';
                        
                    $xml.='</TDadosGNRE>';
                }
                
                $xml.='</guias>
                </TLote_GNRE>
                </gnreDadosMsg>
                </soap12:Body>
                </soap12:Envelope>';
                
        return $xml;
    }
    
    public function formataDataGnre ($data) {
        return (substr($data,6,4).'-'.substr($data,3,2).'-'.substr($data,0,2));
    }
    
    public function getXmlConsultaLoteGnreOnline($row, $parametro){
        $lote    =  $this->factory->getLotegnre($row['NRSEQLOTE']);
        
        $xml ='<?xml version="1.0" encoding="UTF-8"?>
                <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
                    <soap12:Header>
                        <gnreCabecMsg xmlns="http://www.gnre.pe.gov.br/wsdl/consultar">
                            <versaoDados>2.00</versaoDados>
                        </gnreCabecMsg>
                    </soap12:Header>
                    <soap12:Body>
                        <gnreDadosMsg xmlns="http://www.gnre.pe.gov.br/webservice/GnreResultadoLote">
                            <TConsLote_GNRE xmlns="http://www.gnre.pe.gov.br">
                                <ambiente>'.$parametro['IDAMBIENTE'].'</ambiente>
                                <numeroRecibo>'.$lote['NRRECIBO'].'</numeroRecibo>
                            </TConsLote_GNRE>
                        </gnreDadosMsg>
                    </soap12:Body>
                </soap12:Envelope>';
        return $xml;
    }
    
    public function setCertFileKey($row, $parametro) {
        try {
            
            if ($parametro) {
                if (empty($parametro['DSPATHCERTIFICADO'])) {
                    throw new \Exception("Certificado não encontrado. Favor carregar o certificado na tela 'Parametros para Emissão de Guias' para empresa e inscrição selecionadas.");
                }
                if (empty($parametro['DSSENHACERTIFICADO'])) {
                    throw new \Exception("Senha do certificado não encontrada. Favor informar a senha na tela 'Parametros para Emissão de Guias' para empresa e inscrição selecionadas.");
                }
                
                $parametro['DSSENHACERTIFICADO']   = Utilities::zeedhiDecrypt($parametro['DSSENHACERTIFICADO'] );
                //$parametro['CDSENHACERTDIG']   = $this->decrypt($parametro['CDSENHACERTDIG'] );
                
                $this->_certFile    = $this->_fileServerUrl . $parametro['DSPATHCERTIFICADO'];
                $this->_certKeyFile = $this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $row['CDEMPRESA'] . $row['CDINSCESTA'] . ".pem";
                $this->_certPass    = $parametro['DSSENHACERTIFICADO'];
                
                if (fopen($this->_certFile,"r") == true) {
                    if (!file_exists($this->_certKeyFile)) {
                        if (!is_dir($this->_tempCertificatesPath)) {
                            mkdir($this->_tempCertificatesPath, 0777, true);
                        }
                        $data = file_get_contents($this->_certFile);
                        openssl_pkcs12_read($data, $cert, $this->_certPass);
                        
                        if (is_null($cert)) {
                            throw new \Exception("Certificado inválido ou senha inválida");
                        } else {
                            $detail = openssl_x509_parse($cert["cert"], false);
                            if(isset($detail["validTo"])) {
                                if(strtotime(\DateTime::createFromFormat('ymdHis\Z', $detail['validTo'])->format("m/d/Y")) < strtotime(date("m/d/Y"))){ 
                                    throw new \Exception("Certificado expirado.");
                                }
                            }
                            file_put_contents($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $row['CDEMPRESA'] . $row['CDINSCESTA'] . ".pem", $cert["pkey"] . "\r\n" . $cert["cert"]);
                            chmod($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $row['CDEMPRESA'] . $row['CDINSCESTA'] . ".pem", 0777);
                        }
                    
                    } else {
                        $data = file_get_contents($this->_certFile);
                        openssl_pkcs12_read($data, $cert, $this->_certPass);
                        
                        if (is_null($cert)) {
                            throw new \Exception("Certificado inválido ou senha inválida");
                        } else {
                            $detail = openssl_x509_parse($cert["cert"], false);
                            if(isset($detail["validTo"])) {
                                if(strtotime(\DateTime::createFromFormat('ymdHis\Z', $detail['validTo'])->format("m/d/Y")) < strtotime(date("m/d/Y"))){ 
                                    throw new \Exception("Certificado expirado.");
                                }
                            }
                        }
                    }    
                    
                } else {
                    throw new \Exception("Certificado do transmissor não encontrado. Favor carregar o certificado na tela 'Parametros do Reinf'.");
                }
            } else {
                throw new \Exception('Empresa não possui certificado cadastrados. Favor fazer o cadastro na tela Parâmetros de Emissão de Guias.');
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }   
    
    public function trataRetornoEnvioLoteRj ($xmlResult, $rows, $info){
        /*
            O TRATAMENTO DO RETORNO FOI FEITO NO MOMENTO ATULA SOMENTE
            PARA O ENVIO DE UM DOCUMENTO POR VEZ
            ALTERAR NO FUTURO
        */
        
        $xmlDom = $xmlResult['xmlDom'];
        $xml    = $xmlResult['xml'];
        
        $result     = $xmlDom->getElementsByTagName('enviarDadosResponse')->item(0)->getElementsByTagName('enviarDadosResult')->item(0);
        
        $codRetorno = $result->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('CodigoRetorno')->item(0)->nodeValue;
        //$cdErro     = null;
        $msg        = null;
        
        // Caso ocorreu algum erro na transmissao
        if( $xmlDom->getElementsByTagName('Fault')->length > 0 ){
            $tagErro    = $xmlDom->getElementsByTagName('Fault')->item(0);
            $codRetorno =  $tagErro->getElementsByTagName('faultcode')->item(0)->nodeValue;
            $msg        =  $tagErro->getElementsByTagName('faultstring')->item(0)->nodeValue;
                
            return  array ("status"=> false, "mensagem"=>"ERRO -".$codRetorno.' - '.$msg);
        }
        
        else if ($codRetorno != '14') {
            $msg        = $result->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('MensagemRetorno')->item(0)->nodeValue;
            
            return  array ("status"=> false, "mensagem"=>"ERRO NOS DADOS - Código ".$codRetorno.' - '.$msg, "nrseqlote"=>'');
        } 
        else {
            $result     = $xmlDom->getElementsByTagName('enviarDadosResponse')->item(0)->getElementsByTagName('enviarDadosResult')->item(0);
            $recibo     = $xmlDom->getElementsByTagName('enviarDadosResult')->item(0)->getElementsByTagName('IdSessao')->item(0)->nodeValue;   
            $dthrrecibo = $xmlDom->getElementsByTagName('enviarDadosResult')->item(0)->getElementsByTagName('DataRecebimento')->item(0)->nodeValue;   
            $codRetorno = $result->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('CodigoRetorno')->item(0)->nodeValue;
            $msg        = $result->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('MensagemRetorno')->item(0)->nodeValue;
            
            $nrseqlote = $this->newCode->getCodeByTableName('NRSEQLOTE', $this->nrorg);
            
            $this->factory->insereLotegnre($nrseqlote, $info['CDEMPRESA'], $info['CDINSCESTA'], $info['SGESTADO'], 'N', $recibo, $dthrrecibo, $this->nrorg );
            $this->entityManager->flush();

            foreach ($rows as $row) {
                $this->factory->updateTransmissaoDocarrecadpag($row['NRSEQDOCARRECAD'], 'L', $nrseqlote);
                $this->entityManager->flush();
                
                // Apaga lotes sem referência após transmitir
                $this->factory->deleteTranmissaoLote();
                $this->entityManager->flush();
            }
            
            return  array ("status"=> true, "mensagem"=>$msg, "nrseqlote"=>$nrseqlote);
        }
       
    }
    
    public function trataRetornoEnvioLoteGnreOnline ($xmlResult, $rows, $info){
        
        $xmlDom = $xmlResult['xmlDom'];
        $xml    = $xmlResult['xml'];
        
        $result     = $xmlDom->getElementsByTagName('processarResponse')->item(0)->getElementsByTagName('TRetLote_GNRE')->item(0);
        
        $codRetorno = $result->getElementsByTagName('situacaoRecepcao')->item(0)->getElementsByTagName('codigo')->item(0)->nodeValue;
        $descRetorno = $result->getElementsByTagName('situacaoRecepcao')->item(0)->getElementsByTagName('descricao')->item(0)->nodeValue;
        //$cdErro     = null;
        $msg        = null;
        
        // Caso ocorreu algum erro na transmissao
        if( $xmlDom->getElementsByTagName('Fault')->length > 0 ){
            $tagErro    = $xmlDom->getElementsByTagName('Fault')->item(0);
            $codRetorno =  $tagErro->getElementsByTagName('faultcode')->item(0)->nodeValue;
            $msg        =  $tagErro->getElementsByTagName('faultstring')->item(0)->nodeValue;
                
            return  array ("status"=> false, "mensagem"=>"ERRO -".$codRetorno.' - '.$msg);
            
        }
        
        else if ($codRetorno != '100') {
            return  array ("status"=> false, "mensagem"=>"Erro nos dados - Código ".$codRetorno.' - '.$descRetorno, "nrseqlote"=>'');
            
        } 
        else {
            $recibo     = $result->getElementsByTagName('recibo')->item(0);
            $numero     = $recibo->getElementsByTagName('numero')->item(0)->nodeValue;   
            $dthrrecibo     = $recibo->getElementsByTagName('dataHoraRecibo')->item(0)->nodeValue;   
            
            $nrseqlote = $this->newCode->getCodeByTableName('NRSEQLOTE', $this->nrorg);
            
            $this->factory->insereLotegnre($nrseqlote, $info['CDEMPRESA'], $info['CDINSCESTA'], $info['SGESTADO'], 'N', $numero, date_create($dthrrecibo)->format('d/m/Y H:i:s'), $this->nrorg );
            $this->entityManager->flush();

            foreach ($rows as $row) {
                $this->factory->updateTransmissaoDocarrecadpag($row['NRSEQDOCARRECAD'], 'L', $nrseqlote);
                $this->entityManager->flush();
            }
            
            // Apaga lotes sem referência após transmitir
            $this->factory->deleteTranmissaoLote();
            $this->entityManager->flush();
        
            $msg = "Lote transmitido com sucesso. Mensagem de Retorno: ".$codRetorno." - ".$descRetorno;
            
            return  array ("status"=> true, "mensagem"=>$msg, "nrseqlote"=>$nrseqlote);
        }
       
    }
    
    public function trataRetornoConsultaLoteRj ($xmlResult, $lote){
        /*
            O TRATAMENTO DO RETORNO FOI FEITO NO MOMENTO ATULA SOMENTE
            PARA O ENVIO DE UM DOCUMENTO POR VEZ
            ALTERAR NO FUTURO
        */
        
        $this->factory->deleteTranmissaoLote();
        $this->entityManager->flush();
        
        $xmlDom  = $xmlResult['xmlDom'];
        $xml     = $xmlResult['xml'];
        // var_dump($xml);die;
        $falhas  = [];
        
        $codRetorno = null;
        //$cdErro     = null;
        $msg        = null;
        
        $dadosResult =  $xmlDom->getElementsByTagName('consultarDadosResult')->item(0);
        $listaFalha   = $xmlDom->getElementsByTagName('ListaFalha');

        //Retorno do Lote
        $codRetorno  = $dadosResult->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('CodigoRetorno')->item(0)->nodeValue;
        $msg         = $dadosResult->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('MensagemRetorno')->item(0)->nodeValue;
        
        // var_dump($xmlResult);die;
        
        //TRATAR LISTA FALHA
        
        // Sucesso
        /*
            O TRATAMENTO DO RETORNO FOI FEITO NO MOMENTO ATULA SOMENTE
            PARA O ENVIO DE UM DOCUMENTO POR VEZ
            ALTERAR NO FUTURO
        */
        
        if($codRetorno == '0'){
            $documentoRetorno       = $dadosResult->getElementsByTagName('DocsRetorno')->item(0)->getElementsByTagName('DocumentosRetorno')->item(0);
            $itensPagamentoRetorno  = $documentoRetorno->getElementsByTagName('ItensRetorno')->item(0)->getElementsByTagName('ItemPagamentoRetorno')->item(0);
        
            // Retorno do Documento
            $codRetorno      = $itensPagamentoRetorno->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('CodigoRetorno')->item(0)->nodeValue;
            $msg             = $itensPagamentoRetorno->getElementsByTagName('Retorno')->item(0)->getElementsByTagName('MensagemRetorno')->item(0)->nodeValue;
            
            // Sucesso no Item
            if($codRetorno == '0'){
                $docsQuant             = $documentoRetorno->getElementsByTagName('ItensRetorno')->item(0)->childNodes->length;
                $statusLote            =   $listaFalha->item(0)->nodeValue ? 'E' :'S';
                $itensPagamentoRetorno =   $documentoRetorno->getElementsByTagName('ItensRetorno')->item(0)->getElementsByTagName('ItemPagamentoRetorno')->item(0);
                $dataPagamento         =   $documentoRetorno->getElementsByTagName('DataPagamento')->item(0)->nodeValue;
                $nossoNumero           =   $documentoRetorno->getElementsByTagName('NossoNumeroSEFAZ')->item(0)->nodeValue;
                $codigoBarra           =   $documentoRetorno->getElementsByTagName('ListaCodigoBarra')->item(0)->nodeValue;
                $nrLinhaDigitavel      =   self::geraLinhaDigitavel($codigoBarra);
                for($i = 0; $i < $docsQuant; $i++){
                    $itemPagamentoRetorno  = $documentoRetorno->getElementsByTagName('ItensRetorno')->item(0)->getElementsByTagName('ItemPagamentoRetorno')->item($i);
                    $nrseqdocarrecad       = $itemPagamentoRetorno->getElementsByTagName('NumControleContribuinte')->item(0)->nodeValue;
                    $vrICMSPrincipal       =   $itemPagamentoRetorno->getElementsByTagName('ValorICMSPrincipal')->item(0)->nodeValue;
                    $vrJuros               =   $itemPagamentoRetorno->getElementsByTagName('ValorICMSMora')->item(0)->nodeValue;
                    $vrMulta               =   $itemPagamentoRetorno->getElementsByTagName('ValorICMSMulta')->item(0)->nodeValue;

                    $this->factory->updateConsultaLoteDocarrecadpag($nrseqdocarrecad, 'S', $nossoNumero, $codigoBarra, $nrLinhaDigitavel, $vrMulta, $vrJuros, $dataPagamento, $vrICMSPrincipal);
                    $this->entityManager->flush();
                
                }
                
                $this->factory->updateLoteGnre($lote['NRSEQLOTE'], $statusLote);
                $this->entityManager->flush();
                
                if($listaFalha->item(0)->nodeValue){
                    $msg = 'Erro na consulta. Existem pendências.';
                    for($i = 0; $i< $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->length/*)*/; $i++  ){
                        $CoErro = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('CoErro')->item(0)->nodeValue;
                        $DescricaoErro = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('DescricaoErro')->item(0)->nodeValue;    
                        $nrseqdocarrecad = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('NuIdentificador')->item(0)->nodeValue;    
                        $this->factory->insertErroLoteGnre($lote['NRSEQLOTE'], $CoErro, $DescricaoErro, $nrseqdocarrecad, $this->nrorg);
                        $this->factory->updateConsultaLoteDocarrecadpagError($nrseqdocarrecad, 'P');
                        $this->entityManager->flush();
                    }
                }
                $parametrosFiscal = $this->factory->getParametrosFiscal();
                if ($parametrosFiscal['IDGERXMLGNREGUIA'] === 'N') $xml = false; 
            
                return  array ("status"=> false, "mensagem"=> $msg, "xml"=>$xml);
            }
            // Erro no Item
            else{
                if($listaFalha->item(0)->nodeValue){
                    for($i =0; $i< count($listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->length); $i++  ){
                        $msg ='';
                        $msg = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('CoErro')->item(0)->nodeValue;
                        $msg = $msg.' - '. $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('DescricaoErro')->item(0)->nodeValue;    
                        array_push($falhas, $msg);
                    }
                }
                
                $this->factory->updateConsultaLoteDocarrecadpag($nrseqdocarrecad,'C', null, null,null,null, null,null,null);
                $this->entityManager->flush();
            
                $this->factory->updateLoteGnre($lote['NRSEQLOTE'],'C');
                $this->entityManager->flush();
                
                $parametrosFiscal = $this->factory->getParametrosFiscal();
                if ($parametrosFiscal['IDGERXMLGNREGUIA'] === 'N') $xml = false; 
            
                return  array ("status"=> false, "mensagem"=> $msg, "xml"=>$xml);  
            }
        }
        else {
            switch ($codRetorno) {
                case '14':
                    $status = 'R';
                    break;
                    
                case '13':
                case '15':
                    $status = 'P';
                    break;
                default: 
                    $status = 'C';
            }
            
            $docarrecadArray = $this->factory->getDocumentoLote($lote['NRSEQLOTE']);
            
            forEach($docarrecadArray as $docarrecad ) {
                $this->factory->updateConsultaLoteDocarrecadpag($docarrecad['NRSEQDOCARRECAD'], 'P', $docarrecad['NRNOSSONUMERO'], $docarrecad['NRCODIGOBARRA'], $docarrecad['NRLINHADIGI'], $docarrecad['VRMULTA'], $docarrecad['VRJUROS'], $docarrecad['DTLIMITEPAG'], $docarrecad['VRPRINCIPAL']);
                $this->entityManager->flush();
            } 
            
            $this->factory->updateLoteGnre($lote['NRSEQLOTE'], $status);
            $this->entityManager->flush();
            
            if($listaFalha->item(0)->nodeValue){
                $msg = 'Erro na consulta. Existem pendências.';
                for($i =0; $i< $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->length/*)*/; $i++  ){
                    $CoErro = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('CoErro')->item(0)->nodeValue;
                    $DescricaoErro = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('DescricaoErro')->item(0)->nodeValue;    
                    $nrseqdocarrecad = $listaFalha->item(0)->getElementsByTagName('FalhaSessaoLote')->item($i)->getElementsByTagName('NuIdentificador')->item(0)->nodeValue;    
                    $this->factory->insertErroLoteGnre($lote['NRSEQLOTE'], $CoErro, $DescricaoErro, $nrseqdocarrecad, $this->nrorg);
                    $this->entityManager->flush();
                }
            }
            
            $parametrosFiscal = $this->factory->getParametrosFiscal();
            if ($parametrosFiscal['IDGERXMLGNREGUIA'] === 'N') $xml = false; 
            
            return  array ("status"=> false, "mensagem"=> $msg, "xml"=>$xml);
        }
    }
    
    public function trataRetornoConsultaLoteGnreOnline ($xmlResult, $lote){
        
        $this->factory->deleteTranmissaoLote();
        $this->entityManager->flush();
        
        $xmlDom  = $xmlResult['xmlDom'];
        $xml     = $xmlResult['xml'];
        
        $TResultLote     = $xmlDom->getElementsByTagName('gnreRespostaMsg')->item(0)->getElementsByTagName('TResultLote_GNRE')->item(0);
        
        $codRetorno = $TResultLote->getElementsByTagName('situacaoProcess')->item(0)->getElementsByTagName('codigo')->item(0)->nodeValue;
        $descRetorno = $TResultLote->getElementsByTagName('situacaoProcess')->item(0)->getElementsByTagName('descricao')->item(0)->nodeValue;
        
        switch ($codRetorno){
            case '400':
                $status = "R";
                break;
            case '401':
                $status = "P";
                break;
            case '402':
                $status = "S";
                break;
            case '403':
                $status = "E";
                break;
            case '404':
                $status = "C";
                break;
            default:
                $status = "N";
                break; 
        }
        
        $this->factory->updateLoteGnre($lote['NRSEQLOTE'], $status);
        $this->entityManager->flush();
        $results = $TResultLote->getElementsByTagName('resultado')->item(0)->childNodes->length;
        
        if($results > 0) {
            if(in_array($status, ['E','S'])) {    
                for($r = 0; $r < $results; $r++){
                    $guia = $TResultLote->getElementsByTagName('resultado')->item(0)->getElementsByTagName('guia')->item($r);
            
                    $situacaoGuia = $guia->getElementsByTagName('situacaoGuia')->item(0)->nodeValue;
                    $identificadorGuia = $guia->getElementsByTagName('identificadorGuia')->item(0)->nodeValue;
                    
                    switch ($situacaoGuia){
                        case '0':
                            $statusGuia = "S";
                            break;
                        case '1':
                            $statusGuia = "P";
                            break;
                        case '2':
                            $statusGuia = "U";
                            break;
                        case '3':
                            $statusGuia = "C";
                            break;
                        default:
                            $statusGuia = "N";
                            break; 
                    }
                    
                    $docarrecad = $this->factory->getDocumentoPorId($identificadorGuia);
                    
                    $representacaoNumerica = $guia->getElementsByTagName('representacaoNumerica')->length !== 0 ? 
                        $guia->getElementsByTagName('representacaoNumerica')->item(0)->nodeValue : $docarrecad['NRNOSSONUMERO'];
                    $codigoBarras = $guia->getElementsByTagName('codigoBarras')->length !== 0 ? 
                        $guia->getElementsByTagName('codigoBarras')->item(0)->nodeValue : $docarrecad['NRCODIGOBARRA'];
                    $linhaDigitavel = $guia->getElementsByTagName('linhaDigitavel')->length !== 0 ? 
                        $guia->getElementsByTagName('linhaDigitavel')->item(0)->nodeValue : $docarrecad['NRLINHADIGI'];
                    
                    
                    $this->factory->updateConsultaLoteDocarrecadpag($docarrecad['NRSEQDOCARRECAD'], $statusGuia, $representacaoNumerica, $codigoBarras, $linhaDigitavel, $docarrecad['VRMULTA'], $docarrecad['VRJUROS'], $docarrecad['DTLIMITEPAG'], $docarrecad['VRPRINCIPAL']);
                    $this->entityManager->flush();
                    
                    $quantidadeErros        =  $guia->getElementsByTagName('motivosRejeicao')->length !== 0 ?
                        $guia->getElementsByTagName('motivosRejeicao')->item(0)->childNodes->length : 0;
                    
                    for($i = 0; $i < $quantidadeErros; $i++){
                        $motivo                  = $guia->getElementsByTagName('motivosRejeicao')->item(0)->getElementsByTagName('motivo')->item($i);
                        $codigoMotivoRejeicao    = $motivo->getElementsByTagName('codigo')->item(0)->nodeValue;
                        $descricaMotivoRejeicao  = $motivo->getElementsByTagName('descricao')->item(0)->nodeValue;
                        
                        $this->factory->insertErroLoteGnre($lote['NRSEQLOTE'], $codigoMotivoRejeicao, $descricaMotivoRejeicao, $docarrecad['NRSEQDOCARRECAD'], $this->nrorg);
                    }
                }
            }
        }
        
        $parametrosFiscal = $this->factory->getParametrosFiscal();
        if ($parametrosFiscal['IDGERXMLGNREGUIA'] === 'N') $xml = false; 
        
        return  array ("status"=> false, "mensagem"=> 'Resultado: '.$codRetorno.' - '.$descRetorno, "xml"=>$xml);
            
    }
    
}
