<?php
namespace Teknisa\FIS\Services;

use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIS\Factories\Fis08600ParametrosIntegracaoFechamentoCupomFactory;
use Teknisa\Libs\Util\DateUtil;

class Fis08600ParametrosIntegracaoFechamentoCupomService {

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $factory;

    public function __construct(Environment $environment, EntityManager $entityManager, $newCode, Fis08600ParametrosIntegracaoFechamentoCupomFactory $factory) {
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->factory           = $factory;
        $this->nrorg             = $this->environment->getNrOrg();
        $this->operador          = $this->environment->getCdOperador();
        $this->newCode           = $newCode;
        $this->currentTime       = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function verificaFilialParavend($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $filial = $this->factory->verificaFilialParavend($row['CDFILIAL'],$this->nrorg);
            if(empty($filial)){
                $this->factory->insereParavend($row['CDFILIAL'],$this->nrorg,'1');
                $this->entityManager->flush();
                 $connection->commit();
                return true;
            }else{
                $connection->commit();
                return false;
            }

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function deleteParcelamento($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $params = array(
                    "NRORG"         => $this->nrorg,
                    "CDTIPORECE"     => $row["CDTIPORECE"]
                );
                // Busca ultimo Parcelamento para a empresa selecionada.
                $ultimoParcelamento = $this->factory->buscaultimoparcelamento($params);

                // Verifica se o ultimo Parcelamento corresponde ao mapa selecionado.
                if ($ultimoParcelamento['NUMPARCTAXA'] == $row['NUMPARCTAXA']) {
                    
                    $params = array(
                        "NRORG" => $this->nrorg,
                        "NUMPARCTAXA" => $row["NUMPARCTAXA"],
                        "CDTIPORECE"     => $row["CDTIPORECE"]
                    );
                   
                    $this->factory->deletaParcelamento($params);
                      $this->entityManager->flush();
                 $connection->commit();
                return true;
                    
                }else{
                     throw new \Exception("Apenas o último Parcelamento pode ser excluído.");
                }

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
     private function geraProximoSequencialParcelamento() {
        
    }
    
    public function proximoparcelamento($row){

        $result = array();
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $params = array(
                    "NRORG"         => $this->nrorg,
                    "CDTIPORECE"     => $row["CDTIPORECE"]
                    
                );
         // Busca ultimo Parcelamento para a empresa selecionada.
            $ultimoParcelamento = $this->factory->buscaultimoparcelamento($params);
          //  var_dump($ultimoParcelamento);die;
            if($ultimoParcelamento == null){
               $result["NUMPARCTAXA"] = '1';
                
            }else{
                $teste = intval($ultimoParcelamento['NUMPARCTAXA']);
                //var_dump(++ $teste);die;
                $result["NUMPARCTAXA"] = strval(++ $teste);
            }
            
        
         return $result;
    }
    
      public function salvarparcelamento($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

             if($row['__is_new']){
                // var_dump($row);die;
                $this->factory->insereparcelamento($row);
                $this->entityManager->flush();
                 $connection->commit();
            }

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }


    
    public function deleteParcelamentofilial($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $params = array(
                    "NRORG"         => $this->nrorg,
                    "CDTIPORECE"     => $row["CDTIPORECE"],
                    "CDFILIAL"      => $row["CDFILIAL"]
                );
                // Busca ultimo Parcelamento para a empresa selecionada.
                $ultimoParcelamento = $this->factory->buscaultimoparcelamentofilial($params);

                // Verifica se o ultimo Parcelamento corresponde ao mapa selecionado.
                if ($ultimoParcelamento['NUMPARCTAXA'] == $row['NUMPARCTAXA']) {
                    
                    $params = array(
                        "NRORG" => $this->nrorg,
                        "NUMPARCTAXA" => $row["NUMPARCTAXA"],
                        "CDTIPORECE"     => $row["CDTIPORECE"],
                        "CDFILIAL"      => $row["CDFILIAL"]
                    );
                   
                    $this->factory->deletaParcelamentofilial($params);
                      $this->entityManager->flush();
                 $connection->commit();
                return true;
                    
                }else{
                     throw new \Exception("Apenas o último Parcelamento pode ser excluído.");
                }

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function proximoparcelamentofilial($row){

        $result = array();
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $params = array(
                    "NRORG"         => $this->nrorg,
                    "CDTIPORECE"     => $row["CDTIPORECE"],
                    "CDFILIAL"      => $row["CDFILIAL"]
                );
         // Busca ultimo Parcelamento para a empresa selecionada.
            $ultimoParcelamento = $this->factory->buscaultimoparcelamentofilial($params);
          //  var_dump($ultimoParcelamento);die;
            if($ultimoParcelamento == null){
               $result["NUMPARCTAXA"] = '1';
                
            }else{
                $teste = intval($ultimoParcelamento['NUMPARCTAXA']);
                //var_dump(++ $teste);die;
                $result["NUMPARCTAXA"] = strval(++ $teste);
            }
            
        
         return $result;
    }
    
      public function salvarparcelamentofilial($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

             if($row['__is_new']){
                // var_dump($row);die;
                $this->factory->insereparcelamentofilial($row);
                $this->entityManager->flush();
                 $connection->commit();
            }

        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

} 