<?php
namespace Teknisa\FIS\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIS\Factories\Fis22000PreValidacaoDeclaracoesAcessoriasFactory;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Row;

class Fis22000PreValidacaoDeclaracoesAcessoriasService {

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $factory;

    public function __construct(Environment $environment, EntityManager $entityManager, Fis22000PreValidacaoDeclaracoesAcessoriasFactory $factory) {
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->factory           = $factory;
        $this->nrorg             = $this->environment->getNrOrg();
        $this->operador          = $this->environment->getCdOperador();
        $this->currentTime       = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function validarNF($row){
        try {
            $resultado =[];
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            switch ($row['P_DSTIPOVALIDA']) {
                case 'SPED':
                    $resultado = $this->validarSped($row);
                    break;
                case 'SPED_CONTRIB':
                    $resultado = $this->validarSpedContribuicoes($row);
                    break;
                case 'DAPI':
                    $resultado = $this->validarDapi($row);
                    break;
                case 'EFD_Reinf':
                    $resultado = $this->validarEfdReinf($row);
                    break;
            }
            $connection->commit();
            return $resultado;
        }catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function validarSped($row){
        $incricao      = $this->retornaInscricao($row);
        $tipoErroAviso = $this->retornaTipoErroAviso($row);
        $this->factory->validarSped($row['CDIMPOSTO'], $row['CDEMPRESA'], $row['NRSEQUMAPAPU'], $incricao, $row['IDORGARRECAD'], $row['CDLIVROMAG'], $row['IDCONSPRODEST']);
        $notas = $this->factory->buscaNotas($row['CDIMPOSTO'], $row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        $erros = $this->factory->buscaErros($row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        return ( array('notas' => $notas, 'erros'=> $erros)  );
    }

    public function validarDapi($row){
        $incricao      = $this->retornaInscricao($row);
        $tipoErroAviso = $this->retornaTipoErroAviso($row);
        $this->factory->validarDapiMG($row['CDIMPOSTO'], $row['CDEMPRESA'], $row['NRSEQUMAPAPU'], $incricao, $row['IDORGARRECAD'], $row['CDLIVROMAG']);
        $notas = $this->factory->buscaNotas($row['CDIMPOSTO'], $row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        $erros = $this->factory->buscaErros($row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        return ( array('notas' => $notas, 'erros'=> $erros)  );
    }

    public function validarSpedContribuicoes($row){
        $tipoErroAviso = $this->retornaTipoErroAviso($row);
        $this->factory->validarSpedContribuicoes($row['CDEMPRESA'], $row['DATA']['start'], $row['DATA']['end'],$row['CDLIVROMAG']);
        $notas = $this->factory->buscaNotasContribuicoes($row['DATA']['start'], $row['DATA']['end'], $row['CDEMPRESA'], $row['P_ERROS'], $row['P_DSTIPOVALIDA']);
        $erros = $this->factory->buscaErrosContribuicoes($row['CDEMPRESA'], $row['DATA']['start'], $row['DATA']['end'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        return ( array('notas' => $notas, 'erros'=> $erros)  );
    }
    
    public function validarEfdReinf($row){
        $incricao      = $this->retornaInscricao($row);
        $tipoErroAviso = $this->retornaTipoErroAviso($row);
        $this->factory->validarEfdReinf($row['CDIMPOSTO'], $row['CDEMPRESA'], $row['NRSEQUMAPAPU'], $incricao, $row['IDORGARRECAD'], $row['CDLIVROMAG']);
        $notas = $this->factory->buscaNotas($row['CDIMPOSTO'], $row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        $erros = $this->factory->buscaErros($row['NRSEQUMAPAPU'], $tipoErroAviso, $row['P_DSTIPOVALIDA']);
        return ( array('notas' => $notas, 'erros'=> $erros)  );
    }

    public function retornaInscricao ($row){
        switch ($row['IDORGARRECAD']) {
            case 'E':
                $inscricao = $row['CDINSCESTA'];
                break;
            case 'M':
                $inscricao = $row['CDINSCMUNI'];
                break;
            case 'F':
                $inscricao = null;
                break;
            default:
                $inscricao = null;
                break;
        }
        return $inscricao;
    }

    public function retornaTipoErroAviso ($row){
        if($row['P_ERROS']=='T'){
            return null;
        }else{
            return $row['P_ERROS'];
        }
    }


}