var ObrigacoesAcessoriasUtil = function(ScreenService, templateManager, ZHPromise, Messages_fiscal, General, restEngine) {

    // Variável para guardar o escopo atual nas funções.
    var self = this;
    // Variável de controle para o armazenamento da widget atualmente aberta.
    var windowCurrentlyOpen = false;

    /*
     * Função para atualizar um field select conforme necessidade.
     * Caso algum filtro deste select seja alterado, por exemplo, a propriedade
     * "needReload" do field será setada pra true, indicando que o campo precisa
     * ser atualizado. Alterar a propriedade para false depois impede que o campo
     * sempre seja atualizado sem necessidade.
     */
    this.checkFieldNeedReload = function(field) {
        // Verifica se a propriedade não foi setada como false.
        if (field.needReload !== false) {
            // Caso não tenha, recarrega o field e seta a propriedade como false.
            field.reload();
            field.needReload = false;
        }
    };

    /*
     * Função para sair de uma "tela" e retornar à widget principal.
     */
    this.closeSwipe = function(widget) {
        // Prepara uma promise para esta função.
        var deferred = ZHPromise.defer();
        var mainWidget = templateManager.container.getWidget("mainWidget");

        // Desativa a widget atualmente aberta.
        widget.deactivate();

        // Limpa a variável que indica qual a widget atualmente aberta.
        windowCurrentlyOpen = false;

        // Fecha o swipe.
        ScreenService.goBack().then(function() {
            // Ativa a widget principal.
            mainWidget.activate();
            // Finaliza a promise retornada.
            deferred.resolve();
        });
        // Retorna a promise preparada.
        return deferred.promise;
    };

    /*
     * Função que altera a propriedade readOnly do campo de inscrição conforme
     * a inserção de empresa. Também reseta qualquer valor previamente preenchido.
     */
    this.empresaOnChange = function(widget) {
        // Variável booleana que é setada como true caso alguma empresa tenha sido selecionada.
       //blabla
        var empresaIsSelected = !!widget.currentRow.CDEMPRESA;
        var inscricaoField = widget.getField('CDINSCESTA');

        // Altera as propriedades do field de inscrição e reseta seu valor.
        inscricaoField.readOnly = !empresaIsSelected;
        inscricaoField.setValue(null);
        inscricaoField.needReload = true;
    };
    this.empresaOnChangedis = function(widget) {
        // Variável booleana que é setada como true caso alguma empresa tenha sido selecionada.
       //blabla
        var empresaIsSelected = !!widget.currentRow.CDEMPRESADISTR;
        var inscricaoField = widget.getField('CDINSCESTADISTR');

        // Altera as propriedades do field de inscrição e reseta seu valor.
        inscricaoField.readOnly = !empresaIsSelected;
        inscricaoField.setValue(null);
        inscricaoField.needReload = true;
    };

    this.empresaOnChangeInsc = function(widget) {
        // Variável booleana que é setada como true caso alguma empresa tenha sido selecionada.
       //blabla
        var empresaIsSelected = !!widget.currentRow.CDEMPRESA;
        var inscricaoField = widget.getField('CDINSCRICAO');

        // Altera as propriedades do field de inscrição e reseta seu valor.
        inscricaoField.readOnly = !empresaIsSelected;
        inscricaoField.setValue(null);
        inscricaoField.needReload = true;
    };

    /*
     * Função auxiliar para a chamada da função de exportação no backend.
     * Criada para uma padronização nas exportações das telas.
     */
    this.exportarArquivo = function(row, route, message) {
        // Inicializa uma promise.
        var deferred = ZHPromise.defer();
        message = message || Messages_fiscal.EXPORTANDO;

        // Muda a mensagem de carregamento para uma mensagem personalizada.
        ScreenService.changeLoadingMessage(message);

        // Chama a rota para efetuar a exportação.
        restEngine.request(route, {"requestType": "Row", "row": row}, undefined, undefined, undefined, undefined, 99999999999999999999)
        .then(function(response) {
            // Cria a rota a ser chamada para o download do arquivo exportado.
            var downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fis';})[0].baseUrl + "/downloadArquivo?requestType=Row&row[DSARQUIVO]=";
            var fileName = response.dataset.columnsToReturn.fileName;
            
            // Abre a rota em uma nova janela, permitindo que o arquivo seja baixado.
            window.open(downloadRoute + fileName);

            // Finaliza a promise com sucesso.
            deferred.resolve(response);
        })
        .catch(function(error) {
            // Mensagem padrão em caso de timeout.
            var errorMessage = Messages_fiscal.TIMEOUT_ERROR;

            // Verifica se houve alguma resposta da requisição.
            if (error.data && error.data.error) {
                // Caso haja alguma resposta, atribui a mensagem de erro.
                errorMessage = error.data.error;
            }

            // Imprime a mensagem.
            ScreenService.showMessage(errorMessage);

            // Finaliza a promise retornando o objeto de erro.
            deferred.reject(error);
        })
        .finally(function() {
            // Restaura a mensagem original.
            ScreenService.restoreDefaultLoadingMessage();
        });

        // Retorna a promise a ser finalizada.
        return deferred;
    };
    
     this.exportarArquivoGiam = function(row, param, route, message) {
        // Inicializa uma promise.
        var deferred = ZHPromise.defer();
        message = message || Messages_fiscal.EXPORTANDO;

        // Muda a mensagem de carregamento para uma mensagem personalizada.
        ScreenService.changeLoadingMessage(message);

        // Chama a rota para efetuar a exportação.
        restEngine.request(route, {"requestType": "Row", "row": row}, undefined, undefined, undefined, undefined, 99999999999999999999)
        .then(function(response) {
            // Cria a rota a ser chamada para o download do arquivo exportado.
            var downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fis';})[0].baseUrl + "/downloadArquivo?requestType=Row&row[DSARQUIVO]=";
            var fileName = response.dataset.columnsToReturn.fileName;
            
            // Abre a rota em uma nova janela, permitindo que o arquivo seja baixado.
            window.open(downloadRoute + fileName);

            // Finaliza a promise com sucesso.
            deferred.resolve(response);
        })
        .catch(function(error) {
            // Mensagem padrão em caso de timeout.
            var errorMessage = Messages_fiscal.TIMEOUT_ERROR;

            // Verifica se houve alguma resposta da requisição.
            if (error.data && error.data.error) {
                // Caso haja alguma resposta, atribui a mensagem de erro.
                errorMessage = error.data.error;
            }

            // Imprime a mensagem.
            ScreenService.showMessage(errorMessage);

            // Finaliza a promise retornando o objeto de erro.
            deferred.reject(error);
        })
        .finally(function() {
            // Restaura a mensagem original.
            ScreenService.restoreDefaultLoadingMessage();
        });

        // Retorna a promise a ser finalizada.
        return deferred;
    };
    
    this.findLastRowData = function(widgetName, row){
	    return restEngine.post("fis_acessorias#/"+widgetName+"/findLastRowData", {
            "requestType": "Row",
            "row": row
        });
	};

    /*
     * Função para o retorno da data atual.
     */
    this.getCurrentDate = function() {
        return moment().format("DD/MM/YYYY");
    };

    /*
     * Função para o retorno do primeiro dia do mês da data atual.
     */
    this.getFirstDayOfMonth = function() {
        return moment().startOf('month').format("DD/MM/YYYY");
    };

    /*
     * Função para o retorno do último dia do mês da data atual.
     */
    this.getLastDayOfMonth = function() {
        return moment().endOf('month').format("DD/MM/YYYY");
    };

    /*
     * Função que verifica o clique em uma das opções disponíveis na widget
     * principal. Determina se um swipe será aberto ou não, dependendo da
     * opção selecionada.
     */
    this.mainWidgetAfterMoveRow = function(widget) {
        // Guarda o nome da widget relacionada à opção clicada, se houver.
        var windowToOpen = widget.currentRow.WINDOW;
        // Guarda a propriedade "ENABLED" da opção selecionada.
        var windowEnabled = widget.currentRow.ENABLED;

        // Verifica se há uma widget associada (não for um submenu) e se esta
        // está habilitada (a tela estiver disponível para ser utilizada).
        if (windowToOpen && windowEnabled) {
            // Verifica se já há uma widget aberta e se esta é diferente da
            // selecionada.
            if (windowCurrentlyOpen && windowCurrentlyOpen != windowToOpen) {
                // Fecha o swipe com a widget já aberta.
                self.closeSwipe(templateManager.container.getWidget(windowCurrentlyOpen)).then(function() {
                    // Abre o swipe na widget selecionada.
                    ScreenService.openSwipe(templateManager.container.getWidget(windowToOpen));
                });
            }
            // Senão, abre o swipe na opção selecionada.
            else {
                // Abre o swipe na widget selecionada.
                ScreenService.openSwipe(templateManager.container.getWidget(windowToOpen));
            }
            // Altera a variável de controle da widget atualmente aberta.
            windowCurrentlyOpen = windowToOpen;
        }
        // Senão, verifica se não há a propriedade "ENABLED" na opção
        // selecionada, indicando que é um submenu.
        else if (windowEnabled !== false) {
            // Altera o estado aberto/fechado do submenu.
            widget.currentRow._opened = !widget.currentRow._opened;
        }
    };
    
};

Configuration(function(ContextRegister){
    ContextRegister.register('ObrigacoesAcessoriasUtil', ObrigacoesAcessoriasUtil);
});